;ͻ
;Metal Warrior 2 Item handling by Cadaver                                     
;Started in 10/1999!                                                          
;ͼ

ITEM_EMPTY      = 0
ITEM_FISTS      = 0
ITEM_PISTOL     = 1
ITEM_SMG        = 2
ITEM_SHOTGUN    = 3
ITEM_LASER      = 4
ITEM_FLAME      = 5
ITEM_GRENADES   = 6
ITEM_CREDITS2   = 7
ITEM_CREDITS5   = 8
ITEM_CREDITS10  = 9
ITEM_HEALTH     = 10
ITEM_DISK1      = 11
ITEM_DISK2      = 12
ITEM_DISK3      = 13
ITEM_ROPE       = 14
ITEM_DOCS       = 15
ITEM_IDCARD     = 16
ITEM_PILLS      = 17
ITEM_MUSICSHEET = 18
ITEM_COMPUTER   = 19
ITEM_GHETTOBLASTER = 20

IB_DISK1        = 1
IB_DISK2        = 2
IB_DISK3        = 4
IB_ROPE         = 8
IB_DOCS         = 16
IB_IDCARD       = 32
IB_PILLS        = 64
IB_MUSICSHEET   = 128
IB_COMPUTER     = 1
IB_GHETTOBLASTER = 2

IBN_DISK1        = 0
IBN_DISK2        = 1
IBN_DISK3        = 2
IBN_ROPE         = 3
IBN_DOCS         = 4
IBN_IDCARD       = 5
IBN_PILLS        = 6
IBN_MUSICSHEET   = 7
IBN_COMPUTER     = 8
IBN_GHETTOBLASTER = 9

;Ŀ
;FLASHITEMS                                                                   
;                                                                             
;Changes sprite colors of item sprites to make them flash.                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

flashitems:     inc itemflashcnt
                lda itemflashcnt
                lsr
                and #$03
                tax
                lda itemflashtbl,x
                ldx #$10
fli_loop:       sta sprcolortbl+$7f,x
                dex
                bpl fli_loop
                rts

;Ŀ
;AR_ITEM                                                                      
;                                                                             
;Item's actor routine (falling and bouncing)                                  
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

ar_item:        lda actbits,x                   ;Is item in flying motion?
                beq ar_nofly
                lda #4                          ;Perform acceleration,
                ldy #8*8                        ;movement and landing check
                jsr accactory
                jsr moveactory
                lda actsy,x
                jsr getcharpos
                lda actsy,x
                bmi ar_flydone                  ;No landing-check if sy<0
                jsr getcharinfo                 ;Landed?
                and #CI_GROUND
                beq ar_flydone
                lda actyl,x
                and #$f8
                sta actyl,x
                lda actsy,x                     ;Reverse Y-speed
                eor #$ff
                clc
                adc #$10                        ;And add (subtract) it to
                sta actsy,x                     ;prevent endless bouncing
                bmi ar_flydone
                lda #$00
                sta actsy,x
                sta actbits,x                   ;If Y-speed zero, clear
ar_flydone:     rts                             ;flying-bit
ar_nofly:       jsr getcharpos
                jsr getcharinfo
                and #CI_GROUND
                beq ar_setfall
                rts
ar_setfall:     lda #AB_JUMP
                sta actbits,x
                lda #$08
                sta actsy,x
                rts

;Ŀ
;PICKUP                                                                       
;                                                                             
;Player pickup routine, called when ducking.                                  
;                                                                             
;Parameters: X:Actor number 0 (player)                                        
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

pickup:         ldy #MAXACTORS-1
pick_loop:      lda actt,y
                cmp #ACT_ITEM
                beq pick_check
pick_next:      dey
                bne pick_loop
                rts
pick_check:     jsr actorcollide
                bcc pick_next
pick_yes:       lda #FX_PICKUP
                jsr playfx
                lda #$00
                sta actt,y
                lda actf,y                      ;Get type of item
                sta msgnum                      ;Print it!
buyitem:        cmp #ITEM_CREDITS2
                bcc pick_weapon
                cmp #ITEM_CREDITS10+1
                bcc pick_credits
                cmp #ITEM_DISK1
                bcs pick_item
pick_health:    lda acthp
                clc
                adc #HEALTHADD
                cmp #PLRMAXHEALTH+1
                bcc pickhealth_ok
                lda #PLRMAXHEALTH
pickhealth_ok:  sta acthp
                lda #$03
                sta actfls
                rts
pick_credits:   sec
                sbc #ITEM_CREDITS2
                tay
                lda creditstbl,y
                jmp addcredits
pick_weapon:    tay
                sty weapon
                lda weaponaddtbl-1,y            ;Table lookup for how much ammo
                sed                             ;to add
                clc
                adc ammolo,y
                sta ammolo,y
                lda ammohi,y
                adc #$00
                sta ammohi,y
                cld
                lda ammohi,y                    ;Check for ammo over 999
                cmp #$10
                bcc pick_noammo
                lda #$99
                sta ammolo,y
                lda #$09
                sta ammohi,y
pick_noammo:    rts
pick_item:      sec                             ;Picking up items that are
                sbc #ITEM_DISK1                 ;shown in the inventory
                cmp #$03                        ;One of the disks?
                bcs pick_nodisk
                ldy #<diskfoundscript           ;Run the disk picking
                sty csptrlo                     ;cutscene
                ldy #>diskfoundscript
                sty csptrhi
pick_nodisk:    jsr countbit
                lda inventory1
                ora tempval1
                sta inventory1
                lda inventory2
                ora tempval2
                sta inventory2
                rts

countbit:       ldy #$01
                sty tempval1
                dey
                sty tempval2
                tay
                beq countbitdone
countbitloop:   asl tempval1
                rol tempval2
                dey
                bne countbitloop
countbitdone:   rts

;Ŀ
;RANDOM                                                                       
;                                                                             
;Simple random routine                                                        
;                                                                             
;Parameters: -                                                                
;Returns: A:Pseudorandom number                                               
;Modifies: A                                                                  
;

random:         inc randomadd+1
                bne randomseed
                lda randomadd+2
                cmp #>randomareaend-1
                bcc randomok
                lda #>randomareastart-2
randomok:       adc #$01
                sta randomadd+2
randomseed:     lda #$00
randomadd:      eor randomareastart
                adc #$3d
                sta randomseed+1
                rts
