;ͻ
;Metal Warrior 2 - Level routines                                             
;Started in 10/1999!                                                          
;ͼ

MAXLEVELS       = 10
MAXRESTARTPOS   = 12
MAXLVLACT       = 64
NUMDOORS        = 80    ;Important! Change this when the number of doors gets
                        ;bigger, but finddoor will then take more rastertime.

LEVEL_START:

;Ŀ
;LOADSPRITES                                                                  
;                                                                             
;Loads the enemy sprites specified.                                           
;                                                                             
;Parameters: A:sprites number                                                 
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

loadsprites:    cmp loadedsprites               ;Are the sprites already
                beq ls_skipload                 ;in memory?
                sta loadedsprites
                clc
                adc #"0"
                ldx #"S"
                tay
                jsr load
                jsr getenemycolors
ls_skipload:    rts

;Ŀ
;LOADLEVEL                                                                    
;                                                                             
;Loads the level specified.                                                   
;                                                                             
;Parameters: A:level number                                                   
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

loadlevel:      cmp loadedlevel                 ;Is the level already
                beq ll_skipload                 ;in memory?
                sta loadedlevel
                clc
                adc #"0"
                ldx #"L"
                tay
                jsr load
ll_skipload:    lda level
                jsr putleveldata                ;Put old leveldata to storage
                lda loadedlevel
                sta level
                jsr getleveldata                ;And get new level's data from
                ldx level                       ;storage
                lda levelbg1tbl,x               ;Set background colors
                sta r1_setbg1+1
                lda levelbg2tbl,x               ;Set background colors
                sta r1_setbg2+1
                lda levelbg3tbl,x               ;Set background colors
                sta r1_setbg3+1
                lda leveld01btbl,x              ;Set sprite priority
                sta $d01b
                lda levelsprtbl,x               ;And load the correct sprites
                jmp loadsprites

;Ŀ
;LOADLEVELDATA                                                                
;                                                                             
;Loads the initial status of enemies, items etc.                              
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

loadleveldata:  ldx #"S"                        ;Load the initial leveldata-
                ldy #"T"                        ;status
                jsr load
                lda #$ff                        ;Set this to prevent initial
                sta level                       ;corruption of first level
                rts                             ;data

;Ŀ
;PUTLEVELDATA                                                                 
;                                                                             
;Stores the current game level to leveldata storage                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

putleveldata:   lda level                       ;Skip if level undefined
                bmi pld_skip
                clc
                adc #>LEVELDATA
                sta pld_store+2
                ldx #$00
pld_loop:       lda lvlactx,x
pld_store:      sta LEVELDATA,x
                inx
                bne pld_loop
pld_skip:       rts

;Ŀ
;GETLEVELDATA                                                                 
;                                                                             
;Gets the current game level from leveldata storage                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

getleveldata:   lda level                       ;Skip if level undefined
                bmi gld_skip
                clc
                adc #>LEVELDATA
                sta gld_loop+2
                ldx #$00
gld_loop:       lda LEVELDATA,x
                sta lvlactx,x
                inx
                bne gld_loop
gld_skip:       rts

;Ŀ
;ADDACTBORDER                                                                 
;                                                                             
;Adds actors from the leveldata to screen borders. Checks 16 actors at a time 
;to avoid unnecessary CPU use.                                                
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

addactborder:   lda #$00
                sta generatornum
                lda mapx
                tax
                beq aab_nodecx                  ;Prevent X from going negative
                dex
aab_nodecx:     stx tempval1                    ;Left border
                clc
                adc #10
                sta tempval2                    ;Right border
                lda mapy
                tax
                beq aab_nodecy                  ;Same for Y
                dex
aab_nodecy:     stx tempval3                    ;Upper border
                clc
                adc #6
                sta tempval4                    ;Lower border
                lda lvlactcnt                   ;Startpoint for search
                tax
                clc
                adc #$10                        ;Endpoint for search
                sta aab_cmp+1
aab_loop:       lda lvlactt,x                   ;Search for nonzero actor
                bmi aab_generator
                bne aab_notzero
aab_next:       inx
aab_cmp:        cpx #$00
                bcc aab_loop
aab_addcount:   lda aab_cmp+1
                and #$3f
                sta lvlactcnt
                rts
aab_notzero:    lda lvlactx,x                   ;Check all borders
                cmp tempval1
                beq aab_xok1
                cmp tempval2
                bne aab_otherway
aab_xok1:       lda lvlacty,x
                cmp tempval3
                bcc aab_next
                cmp tempval4
                beq aab_ok
                bcc aab_ok
                bcs aab_next
aab_otherway:   lda lvlacty,x
                cmp tempval3
                beq aab_yok1
                cmp tempval4
                bne aab_next
aab_yok1:       lda lvlactx,x
                cmp tempval1
                bcc aab_next
                cmp tempval2
                beq aab_ok
                bcs aab_next
aab_ok:         jsr addactor                    ;If addactor returns C=0,
                bcc aab_addcount                ;there wasn't a free actor
                jmp aab_next                    ;left and we can end our search
aab_generator:  lda lvlacty,x                   ;Check if we are close enough
                cmp mapy                        ;to the generator
                bcc aab_next
                cmp tempval4
                bcs aab_next
                lda lvlactx,x                   ;In X-direction the range is
                clc                             ;30 blocks, so that they don't
                adc #$0a                        ;need to be placed so close to
                cmp mapx                        ;each other
                bcc aab_next
                lda lvlactx,x
                sec
                sbc #$0a
                cmp tempval2
                bcs aab_next
                lda lvlactt,x
                sta generatornum
                jmp aab_next

;Ŀ
;ADDALLACTORS                                                                 
;                                                                             
;Adds actors from the leveldata to screen. Checks all actors, call this when  
;changing position                                                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

addallactors:   lda #$00
                sta lvlactcnt
                lda mapx
                clc
                adc #10
                sta tempval2                    ;Right border
                lda mapy
                clc
                adc #6
                sta tempval4                    ;Lower border
                ldx #$00                        ;Process all actors on level
aaa_loop:       lda lvlactt,x                   ;Search for nonzero actor
                bmi aaa_next                    ;Skip generators now
                bne aaa_notzero
aaa_next:       inx
aaa_cmp:        cpx #MAXLVLACT
                bcc aaa_loop
aaa_done:       rts
aaa_notzero:    lda lvlactx,x                   ;Check the visible area
                cmp mapx
                bcc aaa_next
                cmp tempval2
                bcs aaa_next
                lda lvlacty,x
                cmp mapy
                bcc aaa_next
                cmp tempval4
                bcs aaa_next
                jsr addactor                    ;If addactor returns C=0,
                bcs aaa_next                    ;there wasn't a free actor
                rts                             ;left and we can end our search

addactor:       lda lvlactt,x
                cmp #ACT_ITEM                   ;Item?
                beq adac_gfa2
adac_gfa1:      lda #AN_LASTENEMY+1
                ldy #AN_FIRSTENEMY
                jsr getfreeactor
                bcs adac_found
                rts
adac_gfa2:      lda #AN_LASTITEM+1
                ldy #AN_FIRSTITEM
                jsr getfreeactor
                bcs adac_found
                rts
adac_found:     txa                             ;Store the origin of the actor
                sta actorg,y
                lda #$00                        ;Clear some things
                sta srclo
                sta srchi
                sta actbits,y
                sta actcnt,y
                sta actf,y
                sta actfd,y
                sta actsd,y
                sta actsx,y
                sta actsy,y
                lda lvlactx,x                   ;Multiply X by 32
                jsr mult32srclo
                ora #$10                        ;Add 16
                sta actxl,y
                lda srclo
                sta actxh,y
                lda lvlacty,x                   ;Multiply Y by 32
                jsr mult32srchi
                sta actyl,y
                lda srchi
                sta actyh,y
                lda actxh,y                     ;Check is player to left or
                cmp actxh                       ;right
                beq adac_chklow
                bcs adac_left
                bcc adac_right
adac_chklow:    lda actxl,y
                cmp actxl
                bcs adac_left
adac_right:     lda #$00
                beq adac_facedone
adac_left:      lda #$01
adac_facedone:  sta actd,y
                lda lvlactt,x                   ;And finally, set type
                sta actt,y
                cmp #ACT_BOSS1                  ;Is it a B.O.S.S???
                bcc adac_noboss
                cmp #ACT_BOSS4+1
                bcs adac_noboss
                lda #$01
                sta bossfight
                lda #$00
                sta bossdeath
adac_noboss:    cmp #ACT_ITEM
                beq adac_item
                lda lvlacthp,x                  ;and health
                bne adac_hpok                   ;If health zero, use the
                stx adac_restx+1                ;default health from the
                ldx actt,y                      ;table
                lda acthptbl-ACT_FISTMAN,x
adac_restx:     ldx #$00
adac_hpok:      sta acthp,y
adac_finish:    lda #$00
                sta lvlactt,x                   ;Now remove actor from
                sec                             ;leveldata
                rts
adac_item:      lda lvlacthp,x                  ;Set subtype, in the case of
                sta actf,y                      ;an item
                jmp adac_finish

mult32srclo:    asl                             ;Multiplication routines from
                rol srclo                       ;block to pixel coords - needed
                asl                             ;often!
                rol srclo
                asl
                rol srclo
                asl
                rol srclo
                asl
                rol srclo
                rts

mult32srchi:    asl
                rol srchi
                asl
                rol srchi
                asl
                rol srchi
                asl
                rol srchi
                asl
                rol srchi
                rts

;Ŀ
;GETFREEACTOR                                                                 
;                                                                             
;Subroutine to search for an empty actor. When found, will automatically set  
;the ACTORG to $ff to prevent weird leveldata corruptions + reset hitpoints & 
;flashing                                                                     
;                                                                             
;Parameters: Y:Actor to begin search from                                     
;            A:Actor to end search+1                                          
;Returns: C=1 actor found (Y is number), C=0 not found                        
;Modifies: A,Y                                                                
;

getfreeactor:   sta gfa_cmp+1
gfa_loop:       lda actt,y
                beq gfa_found
                iny
gfa_cmp:        cpy #$00
                bcc gfa_loop
                clc
                rts
gfa_found:      lda #$ff
                sta actorg,y
                lda #$00
                sta acthp,y
                sta actfls,y
                sec
                rts

;Ŀ
;REMOVEACTOR                                                                  
;                                                                             
;Removes an actor from the screen back to leveldata.                          
;                                                                             
;Parameters: X:actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

removeactor:    lda actt,x
                ldy actorg,x                    ;If actor has no origin in
                bmi rma_skip                    ;leveldata, just remove from
                sta lvlactt,y                   ;screen
;                cmp #ACT_ITEM
;                beq rma_skip
;                lda acthp,x                     ;Save hitpoints if they have
;                sta lvlacthp,y                  ;changed
rma_skip:       lda #ACT_EMPTY
                sta actt,x
                rts

;Ŀ
;REMOVEALLACTORS                                                              
;                                                                             
;Removes all actors except player. Use this when changing location.           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

removeallactors:lda #$00
                sta bossfight
                sta bossdeath
                sta newexplosion
                ldx #MAXACTORS-1
raa_loop:       lda actt,x
                beq raa_skip
                jsr removeactor
raa_skip:       dex
                bne raa_loop
                rts

;Ŀ
;REMOVEFARACTORS                                                              
;                                                                             
;Removes actors who are too far off the visible screen.                       
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

removefaractors:lda #$00
                sta srclo
                sta srchi
                lda mapx
                jsr mult32srclo          ;Srclo = X high, Destlo = X low
                sta destlo
                lda mapy
                jsr mult32srchi          ;Srchi = Y high, Desthi = Y low
                sta desthi
                ldx #MAXACTORS-1
rfa_loop:       lda actt,x
                beq rfa_skip
                lda actt,x
                cmp #ACT_BOSS1              ;Bosses won't be removed!
                bcc rfa_noboss
                cmp #ACT_BOSS4+1
                bcc rfa_skip
rfa_noboss:     lda actxl,x
                sec
                sbc destlo
                sta tempval1
                lda actxh,x
                sbc srclo
                beq rfa_xok
                cmp #$01
                beq rfa_xcheck1
                cmp #$ff
                bne rfa_remove
rfa_xcheck2:    lda tempval1
                cmp #<(-64)
                bcs rfa_xok
                bcc rfa_remove
rfa_xcheck1:    lda tempval1
                cmp #<(384)
                bcc rfa_xok
                bcs rfa_remove
rfa_xok:        lda actyl,x
                sec
                sbc desthi
                sta tempval1
                lda actyh,x
                sbc srchi
                beq rfa_ycheck2
                cmp #$ff
                bne rfa_remove
rfa_ycheck1:    lda tempval1
                cmp #<(-48)
                bcs rfa_yok
                bcc rfa_remove
rfa_ycheck2:    cmp #240
                bcc rfa_yok
rfa_remove:     lda actt,x
                jsr removeactor
rfa_yok:
rfa_skip:       dex
                bne rfa_loop
                rts

;Ŀ
;ENEMYGENERATOR                                                               
;                                                                             
;Will generate enemies if a generator was found by addactor routines.         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

enmyg_below:    lda #$00
                sta srclo
                sta srchi
                lda mapy                ;Now get Y-map coordinate
                clc
                adc #$06
                jmp enmyg_above2

enemygenerator: lda generatornum
                bne enmyg_cont
enmyg_skip:     rts
enmyg_cont:     and #$7f                ;Multiply enemygeneratornumber
                sta tempval1            ;by 5
                asl
                asl
                adc tempval1
                sta tempval1
                tax
                jsr random              ;Get random number
                cmp enmygtbl,x          ;Compare to probability value
                bcs enmyg_skip
                jsr random              ;Determine enemy type
                and #$03
                clc
                adc tempval1
                tax
                lda enmygtbl+1,x
                sta tempval4            ;Tempval4 holds the type
                lda #AN_LASTENEMY       ;Now get the free actor if there is
                ldy #AN_FIRSTENEMY      ;any (put only three generated enemies
                jsr getfreeactor        ;on screen at once!)
                bcc enmyg_skip
                lda tempval4
                cmp #ACT_FIREBALL       ;Fireballs spawned from below!
                beq enmyg_below
                cmp #ACT_ROCK           ;Rocks are a speciality: they're
                beq enmyg_above         ;spawned from above!
                lda #$00
                sta srclo
                sta srchi
                jsr random              ;Now decide, which side of screen to
                and #$01                ;use
                sta enmyg_dir+1
                tax
                lda enmygxtbl,x
                clc
                adc mapx
                cmp #$ff                ;If X-map coordinate is negative, skip!
                beq enmyg_skip
                jsr mult32srclo
                ora #$10
                sta actxl,y
                lda srclo
                sta actxh,y             ;X coordinate now ready
                jsr random
                and #$03                ;Now get Y-map coordinate
                sec
                adc mapy
                jsr mult32srchi
                sta actyl,y
                lda srchi
                sta actyh,y             ;Y coordinate now ready
                tya
                tax                     ;Now switch actor index to X
                jsr getcharpos
                jsr getcharinfo
                and #$ff-CI_CLIMB       ;Don't let ladders affect
                ldy tempval4            ;Get actor type
                cmp actbgtbl-ACT_FISTMAN,y ;Compare to bg requirement
                beq enmyg_bgok1
enmyg_skip2:    rts
enmyg_bgok1:    jsr movecharposup1      ;One char above the position
                jsr getcharinfo         ;must be no obstacle
                and #CI_OBSTACLE
                bne enmyg_skip2
enmyg_bgok2:    lda tempval4
                tay
                sta actt,x              ;Now set type
enmyg_dir:      lda #$00
                sta actd,x              ;Set direction
                lda acthptbl-ACT_FISTMAN,y
                sta acthp,x             ;Set hitpoints
                lda #$00                ;Then clear some flags & stuff...
                sta actbits,x
                sta actcnt,x
                sta actf,x
                sta actfd,x
                sta actsd,x
                sta actsx,x
                sta actsy,x             ;And a new enemy is born!
enmyg_aboveskip:rts
enmyg_above:    lda #$00
                sta srclo
                sta srchi
                lda mapy                ;Now get Y-map coordinate
                sec
                sbc #$01
                bcc enmyg_aboveskip     ;If went negative, skip
enmyg_above2:   jsr mult32srchi
                sta actyl,y
                lda srchi
                sta actyh,y             ;Y coordinate now ready
                jsr random              ;Now decide direction for enemy
                and #$01
                sta enmyg_dir+1
                jsr random
                and #$07                ;Then decide X-coordinate
                sec                     ;Add one more deliberately!
                adc mapx
                jsr mult32srclo
                ora #$10
                sta actxl,y
                lda srclo
                sta actxh,y             ;X coordinate now ready
                tya
                tax                     ;Now switch actor index to X
                jmp enmyg_bgok2         ;And skip background check!

;Ŀ
;BGANIM                                                                       
;                                                                             
;Level-specific character animation.                                          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

bganim:         lda level
                bne bganim_notlevel0
                jmp bganim_level0
bganim_notlevel0:
                cmp #$07
                beq bganim_level3
                cmp #$06
                beq bganim_level6
                cmp #$05
                beq bganim_level5
                cmp #$03
                beq bganim_level3
                cmp #$02
                bne bganim_notlevel2
                jmp bganim_level2
bganim_notlevel2:cmp #$09
                bne bganim_donothing
                jmp bganim_level9
bganim_donothing:rts
bganim_level6:  inc bganim_delay
                lda bganim_delay
                and #$03
                bne bganim_level6skip
                ldy GAMECHARS+17*8+4
                lda GAMECHARS+18*8+4
                sta GAMECHARS+17*8+4
                lda GAMECHARS+19*8+4
                sta GAMECHARS+18*8+4
                lda GAMECHARS+20*8+4
                sta GAMECHARS+19*8+4
                sty GAMECHARS+20*8+4
bganim_level6skip:rts
bganim_level5:  inc bganim_delay
                lda bganim_delay
                and #$01
                bne bganim_level6skip
                ldx #$07
bganim_l5loop:  ldy GAMECHARS+155*8,x
                lda GAMECHARS+156*8,x
                sta GAMECHARS+155*8,x
                lda GAMECHARS+157*8,x
                sta GAMECHARS+156*8,x
                tya
                sta GAMECHARS+157*8,x
                dex
                bpl bganim_l5loop
                rts
bganim_level3:  ldx #$07                ;Level 3 - waterfall
bganim_l3loop:  ldy GAMECHARS+117*8,x
                lda GAMECHARS+116*8,x
                sta GAMECHARS+117*8,x
                lda GAMECHARS+115*8,x
                sta GAMECHARS+116*8,x
                tya
                sta GAMECHARS+115*8,x
                dex
                bpl bganim_l3loop
                lda GAMECHARS+118*8+14
                pha
                ldy GAMECHARS+118*8+15
                ldx #$0d
bganim_l3loop2: lda GAMECHARS+118*8,x
                sta GAMECHARS+118*8+2,x
                dex
                bpl bganim_l3loop2
                sty GAMECHARS+118*8+1
                pla
                sta GAMECHARS+118*8
                rts
bganim_level2:  ldx #$03                ;Level 2 - rain
bganim_l2loop:  ldy GAMECHARS+215*8+2,x
                lda GAMECHARS+214*8+2,x
                sta GAMECHARS+215*8+2,x
                lda GAMECHARS+213*8+2,x
                sta GAMECHARS+214*8+2,x
                lda GAMECHARS+212*8+2,x
                sta GAMECHARS+213*8+2,x
                tya
                sta GAMECHARS+212*8+2,x
                dex
                bpl bganim_l2loop
                rts
bganim_level0:  ldx #$04                ;Level 0 - scrolling texts of the
bganim_scloop:  asl GAMECHARS+204*8+2,x ;movie theater
                rol GAMECHARS+203*8+2,x
                rol GAMECHARS+202*8+2,x
                rol GAMECHARS+201*8+2,x
                rol GAMECHARS+200*8+2,x
                rol GAMECHARS+199*8+2,x
                rol GAMECHARS+198*8+2,x
                rol GAMECHARS+197*8+2,x
                dex
                bpl bganim_scloop
                ldy bganim_txtptr
                lda bganim_txt,y
                and #$3f
                bne bganim_nowrap
                sta bganim_txtptr
                rts
bganim_nowrap:  ldx #>TEXTCHARS
                stx srchi
                asl
                asl
                asl
                sta srclo
                bcc bganim_notover
                inc srchi
bganim_notover: ldx bganim_charptr
                lda bganim_andtbl,x
                sta tempval1
                ldx #$04
bganim_charloop:ldy bganim_ytbl,x
                lda (srclo),y
                and tempval1
                beq bganim_sbskip
                lda GAMECHARS+204*8+2,x
                ora #$02
                sta GAMECHARS+204*8+2,x
bganim_sbskip:  dex
                bpl bganim_charloop
                lda bganim_charptr
                clc
                adc #$01
                and #$07
                bne bganim_notover2
                inc bganim_txtptr
bganim_notover2:sta bganim_charptr
                rts
bganim_level9:  inc bganim_delay
                lda bganim_delay
                and #$01
                bne bganim_conveyor
                ldx #$07
bganim_l9loop:  ldy GAMECHARS+155*8,x
                lda GAMECHARS+156*8,x
                sta GAMECHARS+155*8,x
                lda GAMECHARS+157*8,x
                sta GAMECHARS+156*8,x
                tya
                sta GAMECHARS+157*8,x
                dex
                bpl bganim_l9loop
                rts
bganim_conveyor:ldx #$07
bganim_convloop:ldy GAMECHARS+121*8,x
                lda GAMECHARS+177*8,x
                sta GAMECHARS+121*8,x
                tya
                sta GAMECHARS+177*8,x
                ldy GAMECHARS+122*8,x
                lda GAMECHARS+178*8,x
                sta GAMECHARS+122*8,x
                tya
                sta GAMECHARS+178*8,x
                ldy GAMECHARS+123*8,x
                lda GAMECHARS+179*8,x
                sta GAMECHARS+123*8,x
                tya
                sta GAMECHARS+179*8,x
                dex
                bpl bganim_convloop
                rts
