;ͻ
;Metal Warrior 2 music & fx functions                                         
;Started in 8/1999!                                                           
;ͼ

initmusic       = MUSIC
playmusic       = MUSIC+3
stopmusic       = MUSIC+6

MUSIC_TITLE     = 0
MUSIC_HISCORE   = 1
MUSIC_NATURE    = 2
MUSIC_CITY      = 3
MUSIC_CAVES     = 4
MUSIC_INSIDE    = 5
MUSIC_BOSSFIGHT = 6
MUSIC_CUTSCENE  = 7
MUSIC_GAMEOVER  = 8
MUSIC_INTERLUDE = 9
MUSIC_SIREN     = 10
MUSIC_BME       = 11
MUSIC_VICTORY   = 12

FX_NONE         = 0
FX_FOOTSTEP     = 1
FX_JUMP         = 2
FX_HIT          = 3
FX_SHOOT        = 4
FX_PUNCH        = 5
FX_THROW        = 6
FX_LASER        = 7
FX_FLAME        = 8
FX_EXPLOSION    = 9
FX_DEATH        = 10
FX_PICKUP       = 11
FX_SELECT       = 12

SNDMODE_MUSIC   = 0
SNDMODE_SFX     = 1

WAVE_DELAY      = $fe
WAVE_JUMP       = $ff

FREQ_SETSPD     = $fe
FREQ_JUMP       = $ff

MUSIC_START:

;Ŀ
;PLAYLEVELTUNE                                                                
;                                                                             
;Plays the music associated with the current level.                           
;                                                                             
;Parameters: A:Tune number                                                    
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

playleveltune:  ldx level
                lda levelmustbl,x
                cmp currenttune
                beq plt_skip
                jmp playtune
plt_skip:       rts

;Ŀ
;PLAYTUNE                                                                     
;                                                                             
;Starts playing music.                                                        
;                                                                             
;Parameters: A:Tune number                                                    
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

playtune:       sta currenttune
                cmp #MUSIC_NATURE               ;If it is a game-tune, we
                bcc playtune_ok                 ;must check soundmode
                cmp #MUSIC_CUTSCENE+1
                bcs playtune_ok
                ldx soundmode
                beq playtune_ok                 ;Stop music, if a gametune
                lda MUSIC+$66                  ;request while in fx-mode
                beq playtune_noneed
                jsr stopmusic
playtune_noneed:rts
playanytune:
playtune_ok:    pha
                jsr initsfx
                lda #$00
                sta fadeout
                lda #$0f
                sta MUSIC+$36d           ;Reset volume manually
                pla
                sta currenttune
                jmp initmusic

;Ŀ
;FADETUNE                                                                     
;                                                                             
;Starts fade-out of music and waits until complete. Doesn't affect FX!        
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

fadetune:       jsr setfade
waitfade:       jsr waitraster0
                lda MUSIC+$36d
                bne waitfade
                rts
setfade:        lda #$00
                sta fadeoutcount
                lda #$60
                sta fadeout
                sta currenttune
                rts

;Ŀ
;PLAYFX                                                                       
;                                                                             
;Plays a sound effect.                                                        
;                                                                             
;Parameters: A:Sound number                                                   
;Returns: -                                                                   
;Modifies: A                                                                  
;

playfx:         sty playfx_resty+1
                ldy soundmode
                beq playfx_resty
                ldy nextsndchn
                sta sfxchn_newsnd,y
                inc nextsndchn
                lda nextsndchn
                cmp #$03
                bcc playfx_resty
                lda #$00
                sta nextsndchn
playfx_resty:   ldy #$00
                rts

;Ŀ
;TOGGLEFX                                                                     
;                                                                             
;Toggles sound mode, called in gameloop                                       
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

togglefx:       lda soundmode
                eor #$01
                sta soundmode
                beq tgfx_music
tgfx_sound:     jsr initsfx
                jsr stopmusic
                rts
tgfx_music:     ldx #$00
                stx fadeout
                lda currenttune
                jmp playtune_ok

;Ŀ
;INITSFX                                                                      
;                                                                             
;Initializes sound engine.                                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initsfx:        ldx #$02
isfx_loop:      lda #$ff
                sta sfxchn_waveptr,x
                sta sfxchn_freqptr,x
                lda #$00
                sta sfxchn_newsnd,x
                sta sfxchn_wave,x
                dex
                bpl isfx_loop
                sta nextsndchn
                rts

;Ŀ
;DOSFX                                                                        
;                                                                             
;Processes sound effects.                                                     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

dosfx:          ldx #$02
dosfx_loop:     lda sfxchn_newsnd,x
                beq dosfx_notnew
                jmp dosfx_new
dosfx_notnew:   lda sfxchn_wavedly,x
                beq dosfx_nowavedly
                dec sfxchn_wavedly,x
                jmp dosfx_wavedone
dosfx_nowavedly:ldy sfxchn_waveptr,x
dosfx_nwd2:     cpy #$ff
                beq dosfx_storewave
                lda wavetbl,y
                cmp #WAVE_DELAY
                beq dosfx_wdelay
                bcc dosfx_wnormal
dosfx_wjump:    lda wavetbl+1,y
                tay
                jmp dosfx_nwd2
dosfx_wdelay:   iny
                lda wavetbl,y
                sta sfxchn_wavedly,x
                bne dosfx_winc
dosfx_wnormal:  sta sfxchn_wave,x
dosfx_winc:     iny
dosfx_storewave:tya
                sta sfxchn_waveptr,x
dosfx_wavedone: lda sfxchn_freq,x
                clc
                adc sfxchn_freqspd,x
                sta sfxchn_freq,x
                ldy sfxchn_freqptr,x
dosfx_nfd2:     cpy #$ff
                beq dosfx_storefreq
                lda freqtbl,y
                cmp #FREQ_SETSPD
                beq dosfx_fset
                bcc dosfx_fnormal
dosfx_fjump:    lda freqtbl+1,y
                tay
                jmp dosfx_nfd2
dosfx_fset:     iny
                lda freqtbl,y
                sta sfxchn_freqspd,x
                jmp dosfx_finc
dosfx_fnormal:  sta sfxchn_freq,x
dosfx_finc:     iny
dosfx_storefreq:tya
                sta sfxchn_freqptr,x
dosfx_freqdone:
dosfx_regs:     ldy sfxchn_chnindex,x
                lda #$00
                sta $d400,y
                lda sfxchn_freq,x
                sta $d401,y
                lda sfxchn_wave,x
                sta $d404,y
                lda sfxchn_ad,x
                sta $d405,y
                lda sfxchn_sr,x
                sta $d406,y
dosfx_next:     dex
                bmi dosfx_alldone
                jmp dosfx_loop
dosfx_alldone:  lda #$00
                sta $d417
                lda #$0f
                sta $d418
                rts
dosfx_new:      asl
                asl
                adc sfxchn_newsnd,x
                tay
                lda sfxdata-5,y
                sta sfxchn_ad,x
                lda sfxdata-4,y
                sta sfxchn_sr,x
                lda sfxdata-3,y
                sta dosfx_pulsehi+1
                lda sfxdata-2,y
                sta sfxchn_freqptr,x
                lda sfxdata-1,y
                sta sfxchn_waveptr,x
                ldy sfxchn_chnindex,x
                lda #$00
                sta sfxchn_newsnd,x
                sta sfxchn_freqspd,x
                sta sfxchn_wavedly,x
                sta $d404,y
                sta $d405,y
                sta $d406,y
                sta $d402,y
dosfx_pulsehi:  lda #$00
                sta $d403,y
                jmp dosfx_next

