;ͻ
;Metal Warrior 2 - Player routines                                            
;Started in 9/1999!                                                           
;ͼ

PLRMAXHEALTH    = 64
HEALTHADD       = 24
PLAYERDEADTIME  = 75
GAMESTARTPOS    = $11
PASSWORDLENGTH  = 6             ;Number of state variables + 1 (checksum)
CHECKSUMSTART   = $25
CHECKSUMADD     = $7b
BOSS4HEALTH     = 140

PB_BRIEFING     = 1             ;Plot bits: Has Snake given the briefing
PB_IAN1         = 2             ;Found Ian at bottom of pit in lab1?
PB_IAN2         = 4             ;Ian taken to Outpost to recover?
PB_ASYLUM1      = 8             ;Access to cell of Dr.Ultrashred?
PB_ASYLUM2      = 16            ;Dr.Ultrashred given the request for pills?
PB_GOAT         = 32            ;Is Goat in for the recording?
PB_FAUSTUS      = 64            ;Is Faustus in for the recording?
PB_IAN3         = 128           ;Is Ian in for the recording?

PBN_BRIEFING    = 0
PBN_IAN1        = 1
PBN_IAN2        = 2
PBN_ASYLUM1     = 3
PBN_ASYLUM2     = 4
PBN_GOAT        = 5
PBN_FAUSTUS     = 6
PBN_IAN3        = 7

PLAYER_START:

;Ŀ
;INITGAME,INITPLAYER                                                          
;                                                                             
;Performs initialization of player in the beginning of the game.              
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initgame:       lda #$00
                sta bossfight
                sta prevbossfight
                sta csptrlo
                sta csptrhi
                tax
ig_loop1:       sta score,x             ;First zero out everything
                inx
                cpx #(password-score)
                bcc ig_loop1
                lda #$03                ;Then give 3 lives
                sta lives
                lda #$09                ;999 fist-bullets :-)
                sta ammohi
                lda #$99
                sta ammolo
                inc ammohi+1            ;100 pistol-bullets
                lda #$50                ;50 credits
                sta creditslo
                lda #WPN_FISTS
                sta weapon
                lda #$10                ;First extra life at 100000 pts.
                sta extralifecmp
                lda #GAMESTARTPOS
                sta restartpos          ;Start position: level 1 position 0
                rts                     ;(home)

initplayer:     lda #ACT_PLAYERRUN
                sta actt
                lda #$00
                sta actf
                sta actfd
                sta actsd
                sta actcnt
                sta actbits
                sta doortype                    ;Reset door animation/entering
                lda #200                        ;Give some immortality
                sta actfls                      ;in the beginning
                lda #PLRMAXHEALTH
                sta acthp
                rts

;Ŀ
;INITWORLD                                                                    
;                                                                             
;Initializes the game world. (bosses, important items etc.)                   
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initworld:      lda inventory2                  ;Boss 1 alive?
                and #$10
                bne iw_noboss1
                jsr initboss1
                jmp iw_boss1done
iw_noboss1:     lda inventory1                  ;Disk 1 collected?
                and #IB_DISK1
                bne iw_nodisk1
                jsr initdisk1
iw_nodisk1:
iw_boss1done:   lda inventory2                  ;Boss 2 alive?
                and #$20
                bne iw_noboss2
                jsr initboss2
                jmp iw_boss2done
iw_noboss2:     lda inventory1                  ;Disk 2 collected?
                and #IB_DISK2
                bne iw_nodisk2
                jsr initdisk2
iw_nodisk2:     jsr initslime
iw_boss2done:   lda inventory2                  ;Boss 1 alive?
                and #$40
                bne iw_noboss3
                jsr initboss3
                jmp iw_boss3done
iw_noboss3:     lda inventory1                  ;Disk 1 collected?
                and #IB_DISK3
                bne iw_nodisk3
                jsr initdisk3
iw_nodisk3:
iw_boss3done:   lda inventory1                  ;Rope collected?
                and #IB_ROPE
                bne iw_norope
                lda #12
                sta LEVELDATA+$100+$00+63
                lda #9
                sta LEVELDATA+$100+$40+63
                lda #ACT_ITEM
                sta LEVELDATA+$100+$80+63
                lda #14
                sta LEVELDATA+$100+$c0+63
iw_norope:      lda plot                        ;Pills requested by Ultrashred?
                and #PB_ASYLUM2
                beq iw_nopills
                lda inventory1
                and #IB_PILLS                   ;If already got them, don't
                bne iw_nopills                  ;init
                jsr iw_initpills
iw_nopills:     lda plot                  ;Ian in the pit or safe in the
                and #PB_IAN2              ;Outpost?
                beq iw_ianpit
                jsr iw_rescueian
                jmp iw_iandone
iw_ianpit:      lda #3
                sta LEVELDATA+$500+$00+60
                lda #34
                sta LEVELDATA+$500+$40+60
                lda #ACT_IANBASE
                sta LEVELDATA+$500+$80+60
                lda #0
                sta LEVELDATA+$500+$c0+60
iw_iandone:     rts

iw_rescueian:   lda #$00
                sta LEVELDATA+$500+$80+60
                lda #159
                sta LEVELDATA+$100+$00+61
                lda #4
                sta LEVELDATA+$100+$40+61
                lda #ACT_IANRESCUED
                sta LEVELDATA+$100+$80+61
                lda #0
                sta LEVELDATA+$100+$c0+61
                rts

iw_initpills:   lda #135
                sta LEVELDATA+$100+$00+62
                lda #11
                sta LEVELDATA+$100+$40+62
                lda #ACT_ITEM
                sta LEVELDATA+$100+$80+62
                lda #17
                sta LEVELDATA+$100+$c0+62
                rts

initboss1:      lda #18
                sta LEVELDATA+$500+$00+61
                lda #34
                sta LEVELDATA+$500+$40+61
                lda #ACT_BOSS1
                sta LEVELDATA+$500+$80+61
                lda #0
                sta LEVELDATA+$500+$c0+61
                rts
initdisk1:      lda inventory2
                ora #$10
                sta inventory2
                lda #17
                sta LEVELDATA+$500+$00+61
                lda #33
                sta LEVELDATA+$500+$40+61
                lda #ACT_ITEM
                sta LEVELDATA+$500+$80+61
                lda #ITEM_DISK1
                sta LEVELDATA+$500+$c0+61
                rts
initboss2:      lda #10
                sta LEVELDATA+$500+$00+62
                lda #43
                sta LEVELDATA+$500+$40+62
                lda #ACT_BOSS3
                sta LEVELDATA+$500+$80+62
                lda #0
                sta LEVELDATA+$500+$c0+62
                lda #$40       ;When boss2 is alive, slimes are present
                sta slimevalue
                rts
initdisk2:      lda inventory2
                ora #$20
                sta inventory2
                lda #10
                sta LEVELDATA+$500+$00+62
                lda #44
                sta LEVELDATA+$500+$40+62
                lda #ACT_ITEM
                sta LEVELDATA+$500+$80+62
                lda #ITEM_DISK2
                sta LEVELDATA+$500+$c0+62
initslime:      lda #$00       ;After boss2 is gone, slimes are gone too
                sta slimevalue
                rts
initboss3:      lda #12
                sta LEVELDATA+$500+$00+63
                lda #119
                sta LEVELDATA+$500+$40+63
                lda #ACT_BOSS2
                sta LEVELDATA+$500+$80+63
                lda #0
                sta LEVELDATA+$500+$c0+63
                rts
initdisk3:      lda inventory2
                ora #$40
                sta inventory2
                lda #12
                sta LEVELDATA+$500+$00+63
                lda #121
                sta LEVELDATA+$500+$40+63
                lda #ACT_ITEM
                sta LEVELDATA+$500+$80+63
                lda #ITEM_DISK3
                sta LEVELDATA+$500+$c0+63
                rts

;Ŀ
;WARPPLAYER                                                                   
;                                                                             
;Sets player location and level. Then jumps to game mainloop. (so don't JSR)  
;This routine performs quite many actions, like removing actors from screen,  
;storing old leveldata, loading new level, getting new leveldata, adding      
;actors to screen etc.                                                        
;                                                                             
;Parameters: A:Level                                                          
;            X:X position in blocks                                           
;            Y:Y position in blocks                                           
;Returns: -                                                                   
;Modifies: A                                                                  
;

warpplayer:     sta wplr_targetlev+1            ;Store level
                stx wplr_targetx+1
                sty wplr_targety+1
warpplayer2:    jsr removeallactors             ;Remove actors now
                jsr putleveldata                ;And store leveldata
wplr_targetlev: lda #$00
                jsr loadlevel                   ;Load new level
                jsr getleveldata                ;Get new leveldata
                jsr blankgamescreen             ;If there was no loading,
                                                ;blank screen now
                lda mapsizex                    ;Calculate the high limit for
                sec                             ;map X-position
                sbc #$0a
                sta tempval1
                lda #$03
                sta blockx
                lda #$00
                sta blocky
                sta srclo
                sta srchi
                sta actsx                       ;Reset player speed
                sta actsy
wplr_targetx:   lda #$00                        ;Convert X-block to coords
                tax
                jsr mult32srclo
                ora #$10                        ;Align to center of block
                sta actxl
                lda srclo
                sta actxh
wplr_targety:   lda #$00                        ;Convert Y-block to coords
                tay
                jsr mult32srchi
                sta actyl
                lda srchi
                sta actyh
                txa                             ;Then decide the map
                sec                             ;coordinates
                sbc #$05
                bcs wplr_notover1
                lda #$00
                sta blockx
wplr_notover1:  cmp tempval1
                bcc wplr_notover3
                lda #$02
                sta blockx
                lda tempval1
wplr_notover3:  sta mapx
                tya
                sec
                sbc #$04
                bcs wplr_notover2
                lda #$00
wplr_notover2:  sta mapy
                jsr addallactors                ;Add actors on this screen
backtogame:     jsr playleveltune               ;Play level's music
                jsr initbullets                 ;Remove bullets
                jsr initsprites                 ;Remove sprites
                jsr setmappos                   ;Set scrolling & spritescroll
                jsr moveactors
                jsr doactors                    ;Switch new actors & sprites
                jsr dosprites                   ;into position
                jsr switchsprites
                jsr drawmap                     ;And finally draw graphics
                jmp entergameloop

;Ŀ
;FINDACTOR                                                                    
;                                                                             
;Finds the actor the player is talking to. Will set the cutscene pointer      
;and the corresponding cutscene script will be run.                           
;                                                                             
;Parameters: -                                                                
;Returns: C=0 No actor, C=1 actor found.                                      
;Modifies: A,Y                                                                
;

findactor:      ldy #AN_FIRSTENEMY
findactor_loop: lda actt,y                      ;Is it an intelligent actor?
                cmp #ACT_GUNSHOP
                bcc findactor_next
                jsr actorcollide
                bcs findactor_found
findactor_next: iny
                cpy #AN_LASTENEMY
                bcc findactor_loop
                clc
                rts
findactor_found:lda actt,y
                sec
                sbc #ACT_GUNSHOP
                asl
                tay
                lda actorscripttbl,y            ;Run the right script
                sta csptrlo
                lda actorscripttbl+1,y
                sta csptrhi
                sec
                rts


;Ŀ
;FINDDOOR                                                                     
;                                                                             
;Finds the door number that was entered. Charmapx & Charmapy should contain   
;the block where player stands on. If a door was found, will initiate the     
;door-entering animation.                                                     
;                                                                             
;Parameters: A:Door type bit                                                  
;Returns: C=0 No door, C=1 Door found.                                        
;Modifies: A,X (X will become 0 if it wasn't),Y                               
;

finddoor:       sta findd_type+1
                lda bossfight                   ;During bossfight, doors
                ora bossdeath                   ;don't work!!!
                bne findd_skip
                ldy level
                sty findd_cmp1+1
                ldy charmapx
                sty findd_cmp2+1
                ldy charmapy
                dey
                sty findd_cmp3+1
                ldy #NUMDOORS-1
findd_loop:     lda doorl,y                     ;First compare level
findd_cmp1:     cmp #$00
                beq findd_ok1
findd_next:     dey
                bpl findd_loop
findd_skip:     clc
                rts
findd_ok1:      lda doorx,y                     ;Then compare X
findd_cmp2:     cmp #$00
                bne findd_next
                lda doory,y                     ;And finally Y
findd_cmp3:     cmp #$00
                bne findd_next
findd_ok2:      cpy #64                         ;Ultrashred cell door?
                bne findd_notlocked1
                lda plot
                and #PB_ASYLUM1                 ;Should it be open?
                beq findd_skip
findd_notlocked1:cpy #62                        ;Asylum storage door?
                bne findd_notlocked2
                lda plot
                and #PB_ASYLUM2                 ;Should it be open?
                beq findd_skip
findd_notlocked2:
                cpy #72
                bne findd_notfinaldoor
                lda inventory1                  ;Have all disks?
                and #IB_DISK1+IB_DISK2+IB_DISK3
                cmp #IB_DISK1+IB_DISK2+IB_DISK3
                bne findd_skip
                lda inventory2
                and #IB_COMPUTER                ;Have the computer?
                beq findd_skip
                lda #<finaldoorscript           ;Run a "cutscene" :-)
                sta csptrlo
                lda #>finaldoorscript
                sta csptrhi
                jsr runscript
                ldx #$00
                ldy #72
findd_notfinaldoor:sty doornum                     ;We have a match
findd_type:     lda #$00
                sta doortype
                cmp #CI_SIDEDOOR                ;Sidedoor has no animation
                beq findd_noanim
                lda #ACT_PLAYERDOOR
                sta actt
                ldx #$00
                stx dooranimcnt
                stx actf
                stx actfd
                jsr reverseactor                ;Reverse direction for a nice
findd_noanim:   sec                             ;realistic effect
                rts

;Ŀ
;DOORROUTINE                                                                  
;                                                                             
;Performs the animation of doors and moving from one place to another.        
;Might not return with an RTS!                                                
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

doorroutine:    lda pausemode                   ;If paused, do nothing
                bne doorr_quit
                lda doortype
                bne doorr_cont
doorr_quit:     rts
doorr_cont:     ldy actt                        ;If no door-animation, it's a
                cpy #ACT_PLAYERDOOR             ;sidedoor and we will warp
                bne doorr_move                  ;immediately
                ldy actfd
                bne doorr_nowarp
                ldy actf                        ;Warp will happen in the half-
                cpy #$02                        ;way of door animation
                beq doorr_move2
doorr_nowarp:   cmp #CI_MOVINGDOOR              ;If it's a moving door,
                bne doorr_noanim                ;perform animation
                jsr dooranimation
doorr_noanim:   rts
doorr_move:     lda #$00
                sta doortype
doorr_move2:    pla                             ;Remove return address from
                pla                             ;stack
                ldx doornum
                ldy doord,x                     ;Get destination door
                sty doornum                     ;Doornum is now the destination
                ldx doorx,y                     ;Get destination Xpos
                lda doorl,y                     ;Get destination level
                pha
                lda doory,y                     ;Get destination Ypos
                tay
                iny                             ;and add one
                pla
                jmp warpplayer                  ;And off we go!

;Ŀ
;DOORANIMATION                                                                
;                                                                             
;Opening/closing animation of doors.                                          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

dooranimation:  ldy doornum
                lda doorx,y                     ;Get the door's screen position
                sec                             ;(X coordinate)
                sbc mapx
                asl
                asl
                sec
                sbc blockx
                sta destlo
                lda doory,y                     ;Get the door's screen position
                sec                             ;(Y coordinate)
                sbc mapy
                asl
                asl
                sec
                sbc blocky
                sbc #$02                        ;Subtract 2 more (6 chars high)
                cmp #25-6                       ;Abort if the row number is
                bcc danim_yok                   ;highly abnormal
                rts
danim_yok:      tay                             ;Put rownumber to Y
                ldx screennum                   ;Visible screen number
                lda destlo
                clc
                adc screenrowtbllo,y
                sta destlo
                lda screentblhi,x               ;Visible screen address
                adc screenrowtblhi,y
                sta desthi
                lda actfd                       ;Check animation delay
                and #$01
                beq danim_delayok
                rts
danim_delayok:  lda actf                        ;Should it be opening or
                and #$01                        ;closing animation?
                beq danim_open
                jmp danim_close
danim_open:     lda dooranimcnt                 ;Is door already fully open?
                cmp #$02
                bcc danim_openok
                rts
danim_openok:   inc dooranimcnt
                ldx #$05                        ;Row counter
                ldy #$00                        ;Char counter
danim_openloop: lda doorstore2,x
                sta doorstore1,x
                lda (destlo),y
                sta doorstore2,x
                iny                             ;Y=1
                lda (destlo),y
                pha
                lda #$00
                sta (destlo),y
                pla
                dey                             ;Y=0
                sta (destlo),y
                lda doorstore3,x
                sta doorstore4,x
                iny                             ;Y=1
                iny                             ;Y=2
                iny                             ;Y=3
                lda (destlo),y
                sta doorstore3,x
                dey                             ;Y=2
                lda (destlo),y
                pha
                lda #$00
                sta (destlo),y
                pla
                iny                             ;Y=3
                sta (destlo),y
                tya
                clc
                adc #40-3
                tay
                dex
                bpl danim_openloop
                rts
danim_close:    lda dooranimcnt                 ;Is door already fully close?
                bne danim_closeok
                rts
danim_closeok:  dec dooranimcnt
                ldx #$05                        ;Row counter
                ldy #$00                        ;Char counter
danim_closeloop:lda (destlo),y
                pha
                lda doorstore2,x
                sta (destlo),y
                iny                             ;Y=1
                pla
                sta (destlo),y
                lda doorstore1,x
                sta doorstore2,x
                iny                             ;Y=2
                iny                             ;Y=3
                lda (destlo),y
                pha
                lda doorstore3,x
                sta (destlo),y
                pla
                dey                             ;Y=2
                sta (destlo),y
                lda doorstore4,x
                sta doorstore3,x
                tya
                clc
                adc #40-2
                tay
                dex
                bpl danim_closeloop
                rts

;Ŀ
;AR_PLAYERRUN                                                                 
;                                                                             
;Movement routine for player running, jumping, standing.                      
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

ar_playerrun:   lda actbits,x
                and #AB_JUMP
                bne arpr_jump
                jmp arpr_nojump
arpr_jump:      ldy #4
                lda joystick
                and #JOY_RIGHT
                beq arpr_jnotright1
                lda #0
                sta actd,x
                lda #4
                ldy #3*8
                jsr accactorx
                jmp arpr_jxdone
arpr_jnotright1:lda joystick
                and #JOY_LEFT
                beq arpr_jxdone
                lda #1
                sta actd,x
                lda #-4
                ldy #-3*8
                jsr accactorx
arpr_jxdone:    ldy #4
                lda actsy,x
                bpl arpr_nohighjump
                lda actbits,x                   ;When hit, don't allow jumping
                and #AB_HIT                     ;higher
                bne arpr_nohighjump
                lda joystick
                and #JOY_UP
                beq arpr_nohighjump
                ldy #2
arpr_jnocontrol:
arpr_nohighjump:tya
                ldy #8*8
                jsr accactory
                jsr moveactory
                jsr getcharpos
                jsr getrestartpos
                jsr landingcheck
                bcc arpr_jnoland
arpr_landingfx: lda #FX_FOOTSTEP
                jsr playfx
                rts
arpr_jnoland:   jsr checkobst2                  ;In jumps obstacles should
                jsr moveactorx                  ;be checked one char lower
                jsr getcharpos
                jsr landingcheck
                bcs arpr_landingfx
arpr_jumpdone:  lda actf,x                      ;Punching while jumping?
                cmp #11
                bcc arpr_jumpdone2
                lda #2                          ;Punch animation in jump
                jsr animdelay
                bcc arpr_jumpdone2
                inc actf,x
                lda actf,x
                cmp #15
                bcc arpr_jumpdone2
                lda #9
                sta actf,x
arpr_jumpdone2: rts
arpr_nojump:    lda acthp,x
                bne arpr_nodeath
                lda #$00
                sta actcnt,x
                sta actf,x
                lda #ACT_PLAYERDEAD
                sta actt,x
                rts
arpr_nodeath:   jsr getcharpos
                jsr getrestartpos
                jsr getcharinfo
                sta tempval4                    ;For sidedoor check
                and #CI_GROUND
                bne arpr_nofall
                lda #2*8
                sta actsy,x
arpr_initjump:  lda actbits,x
                ora #AB_JUMP
                sta actbits,x
                lda #9
                sta actf,x
                jmp arpr_jump
arpr_nofall:    lda tempval4                    ;If we are standing on a side-
                and #CI_SIDEDOOR                ;door, then warp automatically
                beq arpr_nosidedoor
                jsr finddoor
                bcc arpr_nosidedoor
                rts
arpr_nosidedoor:lda actf,x                      ;Is player punching?
                cmp #11                         ;Then no running/jumping/
                bcc arpr_nopunch                ;ducking possible
                lda #2                          ;Punch animation while
                jsr animdelay                   ;standing
                bcs arpr_punchanim1
                jmp arpr_brake
arpr_punchanim1:inc actf,x
                lda actf,x
                cmp #13
                bcs arpr_punchanim2
                jmp arpr_brake
arpr_punchanim2:jmp arpr_standing
arpr_nopunch:   lda joystick
                and #JOY_UP
                beq arpr_nonewjump
                jsr movecharposup1
                jsr getcharinfo
                sta tempval4                    ;Store for door checking
                dec charmapy
                jsr getcharinfo                 ;But first check ladder
                sta tempval3
                jsr movecharposdown1
                inc charmapy
                lda tempval3
                and #CI_CLIMB
                beq arpr_noclimbup
arpr_initclimb: lda #ACT_PLAYERCLIMB
                sta actt,x
                lda #$00
                sta actf,x
                sta actsx,x
                sta actsy,x
                lda actxl,x
                and #$e0
                ora #$10
                sta actxl,x
                rts
arpr_noclimbup: lda prevjoy
                and #JOY_UP
                bne arpr_nonewjump
                lda tempval4                    ;Now get the doorcheckvalue
                and #CI_OPENDOOR+CI_MOVINGDOOR  ;back
                beq arpr_nodoor
                jsr finddoor                    ;Was a door found? If it was,
                bcc arpr_nodoor                 ;we will warp and can RTS...
                rts
arpr_nodoor:    lda actorpresent                ;Is there an actor nearby?
                beq arpr_noactor
                jsr findactor
                bcc arpr_noactor
                rts
arpr_noactor:   lda #FX_JUMP
                jsr playfx
                lda #-4*8-4
                sta actsy,x
                jmp arpr_initjump
arpr_nonewjump: lda joystick
                and #JOY_DOWN
                beq arpr_noduck
                jsr getcharinfo
                and #CI_CLIMB
                bne arpr_initclimb
                lda #0
                sta actf,x
                lda #ACT_PLAYERDUCK
                sta actt,x
                jsr pickup
                jmp ar_playerduck
arpr_noduck:    lda joystick
                and #JOY_RIGHT
                beq arpr_notright1
                lda #0
                sta actd,x
                lda #8
                ldy #3*8
                jsr accactorx
                jmp arpr_dorunanim
arpr_notright1: lda joystick
                and #JOY_LEFT
                beq arpr_notleft1
                lda #1
                sta actd,x
                lda #-8
                ldy #-3*8
                jsr accactorx
                jmp arpr_dorunanim
arpr_standing:
arpr_notleft1:  lda #0
                sta actf,x
                sta actfd,x
arpr_brake:     lda #4
                jsr brakeactorx
                jmp arpr_dowalk
arpr_dorunanim: lda #2
                jsr animdelay
                bcc arpr_dowalk
                lda actf,x
                and #$03
                bne arpr_nofootstep
                lda #FX_FOOTSTEP
                jsr playfx
arpr_nofootstep:inc actf,x
                lda actf,x
                cmp #9
                bcc arpr_dowalk
                lda #1
                sta actf,x
arpr_dowalk:    jsr checkobst
                jsr moveactorx
                jsr checkstairs
                rts

;Ŀ
;AR_PLAYERDUCK                                                                
;                                                                             
;Movement routine for player ducking.                                         
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

ar_playerduck:  lda joystick
                and #JOY_RIGHT
                beq arpd_notright
                lda #0
                sta actd,x
                jmp arpd_dirdone
arpd_notright:  lda joystick
                and #JOY_LEFT
                beq arpd_dirdone
                lda #1
                sta actd,x
arpd_dirdone:   lda actf,x                      ;Punch animation while ducked?
                beq arpd_nopunch
                lda #2
                jsr animdelay
                bcc arpd_nopunch
                inc actf,x
                lda actf,x
                cmp #3
                bcc arpd_nopunch
                lda #0
                sta actf,x
                jmp arpd_nostand
arpd_nopunch:   lda joystick                    ;Still ducking?
                and #JOY_DOWN
                bne arpd_nostand
                lda #ACT_PLAYERRUN
                sta actt,x
arpd_nostand:   jmp arpr_brake

;Ŀ
;AR_PLAYERCLIMB                                                               
;                                                                             
;Movement routine for player climbing.                                        
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

ar_playerclimb: jsr getcharpos
                lda joystick
                and #JOY_RIGHT
                beq arpc_notright
                lda #$00
                sta actd,x
                jmp arpc_checkleave
arpc_notright:  lda joystick
                and #JOY_LEFT
                beq arpc_notleft
                lda #$01
                sta actd,x
arpc_checkleave:lda joystick                    ;This check prevents
                and #JOY_UP+JOY_DOWN            ;"wobbling" motion when
                bne arpc_noleave                ;pushing the joystick
                jsr getcharinfo                 ;diagonally
                and #CI_GROUND
                beq arpc_noleave
                lda #ACT_PLAYERRUN
                sta actt,x
                jmp dolanding
arpc_noleave:
arpc_notleft:   lda joystick
                and #JOY_UP
                beq arpc_notup
                lda #-4
                ldy #-2*8
                jsr accactory
                jmp arpc_anim
arpc_notup:     lda joystick
                and #JOY_DOWN
                beq arpc_brake
                lda #4
                ldy #2*8
                jsr accactory
                jmp arpc_anim
arpc_brake:     lda #4
                jsr brakeactory
                jmp arpc_anim2
arpc_anim:      lda #3
                jsr animdelay
                bcc arpc_anim2
                lda actf,x
                adc #$00                        ;Carry=1
                and #$03
                sta actf,x
                and #$01
                bne arpc_anim2
                lda #FX_FOOTSTEP
                jsr playfx
arpc_anim2:     jsr checkclimb
                jsr moveactory
                rts
;Ŀ
;AR_PLAYERDEAD                                                                
;                                                                             
;Movement routine for dead player (not much to do :-))                        
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

ar_playerdead:  lda actcnt,x
                cmp #PLAYERDEADTIME
                bcs arpd_done
                inc actcnt,x
arpd_done:      rts

;Ŀ
;AR_PLAYERDOOR                                                                
;                                                                             
;Movement routine for player entering a door.                                 
;                                                                             
;Parameters: X:Actor number                                                   
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

ar_playerdoor:  lda #$06
                jsr animdelay
                bcc arpd_notyet
                inc actf,x
                lda actf,x
                cmp #$04
                bcc arpd_notyet
                lda #$00
                sta actf,x
                sta doortype
                lda #ACT_PLAYERRUN
                sta actt,x
arpd_notyet:    rts

;Ŀ
;CHECKDEATH                                                                   
;                                                                             
;Checks if player has died and do lives remain?                               
;Note: when restarting after a death, doesn't RTS :-)                         
;                                                                             
;Parameters: -                                                                
;Returns: C=0 game continues, C=1 game ends (A=1 to show game over cutscene)  
;Modifies: A                                                                  
;

checkdeath:     lda actt                        ;Has player actor disappeared?
                beq chkd_yes
                cmp #ACT_PLAYERDEAD             ;Dead & deathdelay passed?
                bne chkd_not
                lda actcnt
                cmp #PLAYERDEADTIME
                bcc chkd_not
chkd_yes:       lda cheaton
                bne chkd_unlimited

                sed
                lda lives
                sec
                sbc #$01
                sta lives
                cld
                lda lives
                beq chkd_end

chkd_unlimited: lda bossfight
                bne chkd_boss
                pla
                pla
                jsr fadetune
                lda #$00                        ;Always restart the music!
                sta currenttune
                jmp newlife
chkd_not:       clc
                rts
chkd_end:       lda #$01
                sec
                rts
chkd_boss:      jsr initplayer                  ;In a bossfight, new life
                clc                             ;starts immediately from the
                rts                             ;location

;Ŀ
;NEWLIFE                                                                      
;                                                                             
;Initializes player to restartpos. Isn't a subroutine but something to be     
;JMPed to...                                                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

newlife:        jsr initplayer
                ldx restartpos                  ;Even-numbered restart-
                txa                             ;positions have the player
                and #$01                        ;facing right.
                sta actd
                txa
                lsr                             ;Extract level from the
                lsr                             ;position
                lsr
                lsr
                sta newlife_level+1
                jsr getrestarttbl
                txa                             ;Then get position within
                and #$0f                        ;level
                asl
                tay
                lda (srclo),y
                tax                             ;X position
                iny
                lda (srclo),y
                tay                             ;Y position
newlife_level:  lda #$00
                jmp warpplayer
getrestarttbl:  asl
                tay
                lda lvlrestarttbl,y             ;Get restarttable address
                sta srclo
                lda lvlrestarttbl+1,y
                sta srchi
                rts

;Ŀ
;GETRESTARTPOS                                                                
;                                                                             
;Checks if player has entered a new restart position.                         
;Is called from within the player move routine and uses charmapx & charmapy!  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,Y                                                                
;

getrestartpos:  lda charmapx
                sta grp_checkx+1
                lda charmapy
                sta grp_checky1+1
                clc
                adc #$02
                sta grp_checky2+1
                lda level
                jsr getrestarttbl
                ldy #MAXRESTARTPOS*2-2
grp_loop:       lda (srclo),y                   ;Check X
grp_checkx:     cmp #$00
                beq grp_xok
grp_next:       dey
                dey
                bpl grp_loop
                rts
grp_xok:        iny
                lda (srclo),y                   ;Check Y
                dey
grp_checky1:    cmp #$00
                bcc grp_next
grp_checky2:    cmp #$00
                bcs grp_next
grp_found:      tya
                lsr
                sta tempval1
                lda level
                asl
                asl
                asl
                asl
                adc tempval1
                sta restartpos
                rts

;Ŀ
;CHECKSCROLL                                                                  
;                                                                             
;Checks whether screen should be scrolled.                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

checkscroll:    lda actxl
                sec
                sbc scrollsubxl
                sta destlo
                lda actxh
                sbc scrollsubxh
                sta desthi
                lda destlo
                sec
                sbc #182
                sta destlo
                bcs chkscr_notover1
                dec desthi
chkscr_notover1:lda desthi
                bmi chkscr_xneg
chkscr_xpos:    lda destlo
                jsr chkscr_divide
                sta scrollsx
                jmp chkscr_y
chkscr_xneg:    lda destlo
                eor #$ff
                clc
                adc #$01
                jsr chkscr_divide
                eor #$ff
                clc
                adc #$01
                sta scrollsx
chkscr_y:       lda actyl
                sec
                sbc scrollsubyl
                sta destlo
                lda actyh
                sbc scrollsubyh
                sta desthi
                lda destlo
                sec
                sbc #181
                sta destlo
                bcs chkscr_notover2
                dec desthi
chkscr_notover2:lda desthi
                bmi chkscr_yneg
chkscr_ypos:    lda destlo
                jsr chkscr_divide2
                sta scrollsy
                rts
chkscr_yneg:    lda destlo
                eor #$ff
                sec
                sbc #29
                bcc chkscr_yzero
                jsr chkscr_divide2
                eor #$ff
                clc
                adc #$01
                sta scrollsy
                rts
chkscr_yzero:   lda #$00
                sta scrollsy
                rts
chkscr_divide:  lsr
                lsr
                lsr
                lsr
                cmp #$09
                bcc chkscr_nothigh
                lda #$08
chkscr_nothigh: rts
chkscr_divide2: lsr
                cmp #$09
                bcc chkscr_nothigh2
                lda #$08
chkscr_nothigh2:rts

;Ŀ
;UPDATESTATUS                                                                 
;                                                                             
;Draws the changed parts of status.                                           
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

updatestatus:   lda score
                cmp prevscore
                bne upds_drawscore
                lda score+1
                cmp prevscore+1
                bne upds_drawscore
                lda score+2
                cmp prevscore+2
                bne upds_drawscore
                lda score+3
                cmp prevscore+3
                beq upds_skipscore
upds_drawscore: jsr drawscore
upds_skipscore: lda acthp
                cmp prevhp
                beq upds_skiphealth
                jsr drawhealth
upds_skiphealth:lda lives
                cmp prevlives
                beq upds_skiplives
                jsr drawlives
upds_skiplives: lda weapon
                cmp prevweapon
                beq upds_skipweapon
                jsr setweapon
upds_skipweapon:ldx weapon
                lda ammolo,x
                cmp prevammolo
                bne upds_drawammo
                lda ammohi,x
                cmp prevammohi
                beq upds_skipammo
upds_drawammo:  jsr drawammo
upds_skipammo:  lda creditslo
                cmp prevcreditslo
                bne upds_drawcredits
                lda creditshi
                cmp prevcreditshi
                beq upds_skipcredits
upds_drawcredits:jsr drawcredits
upds_skipcredits:lda explo
                cmp prevexplo
                bne upds_drawexp
                lda explevel
                cmp prevexplevel
                beq upds_skipexp
upds_drawexp:   jsr drawexp
upds_skipexp:   lda inventory1
                cmp previnventory1
                bne upds_drawitems
                lda inventory2
                cmp previnventory2
                beq upds_skipitems
upds_drawitems: jsr drawitems
upds_skipitems: rts

;Ŀ
;Subroutines for drawing the status.                                          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

drawscore:      ldx #$03
                ldy #$00
drscore_loop:   lda score,x
                sta prevscore,x
                lsr
                lsr
                lsr
                lsr
                clc
                adc #$30
                sta TEXTSCREEN+21*40+1,y
                iny
                lda score,x
                and #$0f
                clc
                adc #$30
                sta TEXTSCREEN+21*40+1,y
                iny
                dex
                bpl drscore_loop
                rts

drawhealth:     ldy #$00
                lda acthp
                sta prevhp
                lsr
                lsr
                lsr
                tax
                beq drhlth_skip1
                lda #$43
drhlth_loop1:   sta TEXTSCREEN+23*40+1,y
                iny
                dex
                bne drhlth_loop1
drhlth_skip1:   lda acthp
                lsr
                and #$03
                beq drhlth_loop2
                clc
                adc #$3f
                sta TEXTSCREEN+23*40+1,y
                iny
drhlth_loop2:   cpy #$08
                bcs drhlth_done
                lda #$20
                sta TEXTSCREEN+23*40+1,y
                iny
                bne drhlth_loop2
drhlth_done:    rts

drawlives:      lda lives
                sta prevlives
                ldy #10
                jsr drawnumbers
                ldy tempval2
                rts

drawammo:       ldx weapon
                lda ammohi,x
                sta prevammohi
                and #$0f
                clc
                adc #$30
                sta TEXTSCREEN+23*40+15
                lda ammolo,x
                sta prevammolo
                ldy #16
                jsr drawnumbers
                rts

drawcredits:    lda creditshi
                sta prevcreditshi
                ldy #22
                jsr drawnumbers
                lda creditslo
                sta prevcreditslo
                ldy #24
                jsr drawnumbers
                rts

drawexp:        lda explo
                sta prevexplo
                ldy #32
                jsr drawnumbers
                lda explevel
                sta prevexplevel
                and #$0f
                clc
                adc #$31
                sta TEXTSCREEN+23*40+38
                rts

drawnumbers:    pha
                lsr
                lsr
                lsr
                lsr
                clc
                adc #$30
                sta TEXTSCREEN+23*40,y
                pla
                and #$0f
                clc
                adc #$30
                sta TEXTSCREEN+23*40+1,y
                rts

drawitems:      lda msgtime
                bne drit_quit
                lda inventory1
                sta previnventory1
                sta tempval1
                lda inventory2
                sta previnventory2
                sta tempval2
                ldx #$09
                ldy #20
drit_loop:      lda #$20
                lsr tempval2
                ror tempval1
                bcc drit_skip
                txa
                adc #91                         ;Add one less because C=1
drit_skip:      sta TEXTSCREEN+21*40,y
                iny
                cpy #39
                bcs drit_nospace
                lda #$20                        ;If not in the border, draw
                sta TEXTSCREEN+21*40,y          ;a space between items
                iny
drit_nospace:   dex
                bpl drit_loop
drit_quit:      rts

;Ŀ
;ADDSCORE                                                                     
;                                                                             
;Adds srclo,srchi to the score.                                               
;                                                                             
;Parameters: Srclo,Srchi                                                      
;Returns: -                                                                   
;Modifies: A                                                                  
;

addscore:       sed
                lda score
                clc
                adc srclo
                sta score
                lda score+1
                adc srchi
                sta score+1
                lda score+2
                adc #$00
                sta score+2
                lda score+3
                adc #$00
                sta score+3
                lda extralifecmp             ;Check for extra lives at 100000's
                cmp score+2
                bne adsc_nolife
                clc
                adc #$10
                sta extralifecmp
                jmp addlife
adsc_nolife:    cld
                rts

plotreward:     lda #$00                    ;Give bonus of 10000 points for
                sta srclo                   ;advance in the plot
                lda #$50
                sta srchi
                jsr addscore
                jsr addscore
                jmp updatestatus
;Ŀ
;ADDLIFE                                                                      
;                                                                             
;Adds one extra life.                                                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

addlife:        sed
                lda lives
                clc
                adc #$01
                bcc adlf_notover
                lda #$99
adlf_notover:   sta lives
                cld
                rts

;Ŀ
;ADDEXP                                                                       
;                                                                             
;Adds experience to player, going up a level if necessary.                    
;                                                                             
;Parameters: A:Experience given                                               
;Returns: -                                                                   
;Modifies: A                                                                  
;

addexp:         sed
                clc
                adc explo
                sta explo
                cld
                bcc addexp_nolev
addexp_notover: lda explevel    ;If experience level 9, can't go higher!
                cmp #$08
                bcc addexp_levelok
                lda #$99
                sta explo
                rts
addexp_levelok: inc explevel
                lda #25
                sta msgnum
addexp_nolev:   rts

;Ŀ
;ADDCREDITS                                                                   
;                                                                             
;Adds credits to player.                                                      
;                                                                             
;Parameters: A:How many credits                                               
;Returns: -                                                                   
;Modifies: A                                                                  
;

addcredits:     sed
                clc
                adc creditslo
                sta creditslo
                lda creditshi
                adc #$00
                sta creditshi
                cld
                bcc addcredits_ok
                lda #$99
                sta creditslo
                sta creditshi
addcredits_ok:  rts

;Ŀ
;CALCPASSWORD                                                                 
;                                                                             
;Calculates password after game has ended.                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

calcpassword:   lda restartpos                  ;Get level number
                and #$f0
                cmp #$50                        ;If it's the lab level,
                bne cpwd_notlevel5              ;start always from beginning
                lda restartpos                  ;of each lab
                and #$fc
                sta restartpos
cpwd_notlevel5: lda restartpos                  ;If the main complex, start
                and #$f0                        ;always from beginning
                cmp #$90
                bne cpwd_notlevel9
                lda #$90
                sta restartpos
cpwd_notlevel9: ldx #PASSWORDLENGTH-2
                lda #CHECKSUMSTART              ;Initialize checksum
                sta tempval1
cpwd_loop1:     lda explevel,x                  ;Copy state variables to
                sta password+1,x                ;password
                sec
                sbc tempval1
                sta tempval1                    ;And subtract from checksum
                dex
                bpl cpwd_loop1
                sta password
                ldx #PASSWORDLENGTH-2
cpwd_loop2:     lda tempval1
                clc
                adc #CHECKSUMADD
                sta tempval1
                eor password+1,x                ;Now EOR the rest of the
                sta password+1,x                ;password
                dex
                bpl cpwd_loop2
                rts

;Ŀ
;CHECKPASSWORD                                                                
;                                                                             
;Checks that the user's password is valid.                                    
;                                                                             
;Parameters: -                                                                
;Returns: C=0 invalid, C=1 valid (copied to game state variables)             
;Modifies: A,X,Y                                                              
;

checkpassword:  lda password                    ;Get checksum
                sta tempval1
                ldx #PASSWORDLENGTH-2
chkpwd_loop1:   lda tempval1
                clc
                adc #CHECKSUMADD
                sta tempval1
                eor password+1,x                ;Now un-EOR the password to the
                sta explevel,x                  ;state variables
                dex
                bpl chkpwd_loop1
        ;Then calculate checksum and compare
                ldx #PASSWORDLENGTH-2
                lda #CHECKSUMSTART              ;Initialize checksum
                sta tempval1
chkpwd_loop2:   lda explevel,x
                sec
                sbc tempval1
                sta tempval1                    ;And subtract from checksum
                dex
                bpl chkpwd_loop2
                lda tempval1                    ;Compare to the checksum in
                cmp password                    ;the user's password
                bne chkpwd_fail
        ;Here are the checks about highly invalid state variables,
        ;like level=$ff :-)
                lda explevel                    ;Experience level > 9
                cmp #$09
                bcs chkpwd_fail
                lda restartpos
                cmp #$10*MAXLEVELS              ;Illegal startpoint?
                bcs chkpwd_fail
                and #$0f
                cmp #MAXRESTARTPOS
                bcs chkpwd_fail
                sec
                rts
chkpwd_fail:    clc
                rts
