;ͻ
;Metal Warrior 2 - Raster Interrupt routines                                  
;Started in 8/1999!                                                           
;ͼ

RASTER0POS      = 205
RASTER1POS      = 16
RASTER2POS      = 54
RASTER4POS      = 125

RASTER_START:

;Ŀ
;INITRASTER                                                                   
;                                                                             
;Initializes cycling of raster interrupts and switches KERNAL off.            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initraster:     sei
                lda #$35                        ;Set all ROM off, IO on.
                sta $01
                lda #<idleirq
                sta $0314
                lda #>idleirq
                sta $0315
                lda #<nmi                       ;Set NMI, both the hardware
                sta $0318                       ;vector and the vector used by
                sta $fffa                       ;KERNAL
                lda #>nmi
                sta $0319
                sta $fffb
                lda #<raster                    ;Set main IRQ vector
                sta $fffe
                lda #>raster
                sta $ffff
                lda #$7f                        ;Set timer interrupt off
                sta $dc0d
                lda #$01                        ;Set raster interrupt on
                sta $d01a
                lda #27                         ;Set high bit of raster
                sta $d011                       ;position (0)
                lda #<raster1                   ;Set IRQ vector for first
                sta $fffe                       ;raster interrupt
                lda #>raster1
                sta $ffff
                lda #RASTER1POS                 ;Set low bits of position
                sta $d012                       ;for first raster interrupt
                lda $dc0d                       ;Acknowledge timer interrupt
                cli                             ;(for safety)
                rts

idleirq:        inc $d019
                jmp $ea81

;Ŀ
;RASTER                                                                       
;                                                                             
;Main raster routine, which saves register values etc. before jumping into    
;the correct raster code.                                                     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster:
endraster:      inc $d019                       ;Acknowledge IRQ
                lda rasterlo,x
                sta $fffe
                lda rasterhi,x
                sta $ffff
                lda rasterpos,x
                sta $d012
                lda savea
                ldx savex
                ldy savey
nmi:            rti

;Ŀ
;RASTER0                                                                      
;                                                                             
;Raster interrupt at the bottom of the screen, which initializes the score    
;screen. Plays also music!                                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster0:        cld
                sta savea
                lda $d011
                cmp #$15
                beq r0_badline
                lda #$1f
                sta $d011
                pha
                pla
                pha
                pla
                nop
                nop
                nop
                nop
r0_badline:     lda #$5f                        ;Blank screen
                sta $d011
                lda $dd00                       ;Calculate videobank
                ora #$01
                sta $dd00                       ;Set videobank
                lda #$98                        ;Set screen and character set
                sta $d018
                stx savex
                sty savey
                lda #$00                        ;Set background color
                sta $d021
                lda #$09                        ;Set multicolors for the score
                sta $d022                       ;screen
                lda #$0a
                sta $d023
r0_wait2:       lda $d012
                cmp #RASTER0POS+2
                bcc r0_wait2
                lda #$18                        ;Set X-scrolling
                sta $d016
                lda screenmode                  ;Is it the end scroll?
                cmp #SCRMODE_END
                bne r0_nolongwait
r0_longwait:    lda $d012                       ;Wait longer in that case
                cmp #RASTER0POS+10
                bcc r0_longwait
r0_nolongwait:  lda #$1f                        ;Now switch screen on
                sta $d011
                lda MUSIC+$66                   ;Music or sfx?
                beq r0_sfx
                jsr playmusic                   ;Play the evil symphony
                jmp r0_snddone
r0_sfx:         jsr dosfx                       ;Play the evil sounds
r0_snddone:     lda fadeout
                beq r0_nofade
                clc
                adc fadeoutcount
                sta fadeoutcount
                bcc r0_nofade
                lda MUSIC+$36d
                beq r0_nofade
                dec MUSIC+$36d
r0_nofade:      inc r0count                     ;Increase raster counter
                ldx #$01                        ;Number of next raster int.
                jmp endraster

settextscreen:  lda #$00
                sta $d015                       ;Set all sprites off
                lda #$0b                        ;Set multicolors
                sta $d022
                lda #$05
                sta $d023
                lda $dd00                       ;Set videobank
                and #$fc
                ora #$01
                sta $dd00
                lda #$98                        ;Set screen and character set
                sta $d018
                lda #$18                        ;Set X-scrolling
                sta $d016
                lda #$00                        ;Set background color
                sta $d021
                rts



;Ŀ
;RASTER1                                                                      
;                                                                             
;Raster interrupt at the top of the screen, which initializes the game/title  
;screen.                                                                      
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster1:        cld                             ;Clear decimal mode
                sta savea
                stx savex
                sty savey
                inc r1count                     ;Increase raster counter
                ldy screenmode                  ;Branch according to screenmode
                beq r1_game
                cpy #SCRMODE_CUTS
                beq r1_cuts
                cpy #SCRMODE_END
                bne r1_text
                lda scrolly
                and #$07
                ora #$10
                sta $d011
                bne r1_endtext
r1_text:        lda #$17
                sta $d011
r1_endtext:     jsr settextscreen
                ldx #$00                        ;Do we need the logo raster-
                cpy #SCRMODE_LOGO               ;effect?
                bne r1_end2
                ldx #$02                        ;Number of next raster int.
r1_end2:        jmp endraster
r1_cuts:        lda #$18
                sta $d016
                lda #$57                        ;In cutscenes use the blanking-
                sta $d011                       ;mode!
                lda #$0b                        ;Set sprite multicolor 1
                bne r1_gamecommon
r1_game:        lda scrollx                     ;Set game scrolling
                and #$07
                ora #$10
                sta $d016
                lda scrolly
                and #$07
                ora #$10
                sta $d011
                lda #$00                        ;Set sprite multicolor 1
r1_gamecommon:  sta $d025
                lda $dd00                       ;Set game videobank
                and #$fc
                sta $dd00
                ldx screennum                   ;Set screen & character set
                lda r1_d018tbl,x                ;according to screennum
                sta $d018
r1_setbg1:      lda #$00                        ;Set multicolors for the game
                sta $d021                       ;screen
r1_setbg2:      lda #$00
                sta $d022
r1_setbg3:      lda #$00
                sta $d023
                ldx #$00                        ;Now check which half of
                lda r1sprirqturn                ;the sprirq-doublebuffer
                and #$7f
                sta r1sprirqturn
                beq r1_notturn2
                ldx #MAXSPRIRQ
r1_notturn2:    stx r1sprirqnum
                ldy sprirqamount,x              ;Any sprites?
                lda r1_d015tbl,y                ;Switch the necessary sprites
                sta $d015                       ;on
                beq r1_end
                ldy screennum
                lda r1_frametbl,y               ;Put the correct screen address
                sta r3_loadf8+2                 ;to the load routine
                sta r3_loadf7+2
                sta r3_loadf6+2
                sta r3_loadf5+2
                sta r3_loadf4+2
                sta r3_loadf3+2
                sta r3_loadf2+2
                sta r3_loadf1+2
                jmp r3_direct
r1_end:         tax                             ;Cutscene split?
                lda screenmode
                cmp #SCRMODE_CUTS
                bne r1_end3
                ldx #$04
r1_end3:        jmp endraster

;Ŀ
;RASTER2                                                                      
;                                                                             
;Raster interrupt for MW2 logo color effect.                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster2:        cld                             ;Clear decimal mode
                sta savea
                stx savex
                sty savey
                ldx #19                         ;Initialize color index
                ldy #$00                        ;Initialize badline counter
                pha
                pla
                pha
                pla
                nop
                nop
r2_loop:        lda r2_colors,x         ;4
                sta $d023               ;8
                dey                     ;10
                bmi r2_bad              ;12
                REPEAT 6
                pha
                pla
                REPEND
                nop
                nop
                dex                     ;14
r2_jump1:       bpl r2_loop             ;17
                inx
                jmp endraster
r2_bad:         ldy #7                  ;15
                dex                     ;17
r2_jump2:       bpl r2_loop             ;20
                inx
                jmp endraster

;Ŀ
;RASTER3                                                                      
;                                                                             
;Raster interrupt for sprite multiplexing.                                    
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster3:        cld                             ;Clear decimal mode
                sta savea
                stx savex
                sty savey
                ldx r1sprirqnum
r3_direct:      ldy sprirqamount,x
                lda r3_loadytbllo-1,y
                sta r3_jmp1+1
                lda r3_loadytblhi-1,y
                sta r3_jmp1+2
                lda r3_loadrtbllo-1,y
                sta r3_jmp2+1
                lda r3_loadrtblhi-1,y
                sta r3_jmp2+2
                ldy sprirqstart,x
r3_jmp1:        jmp $0000
r3_sprdone:     ldx r1sprirqnum
                inx                             ;Off to the next irq
                lda sprirqamount,x              ;If zero sprites, it was last
                beq r3_last
r3_notlast:     stx r1sprirqnum
                lda $d012                       ;Are we late from the next
                clc                             ;spriteirq?
                adc #$01
                cmp sprirqline,x
                bcs r3_direct                   ;Yes, jump direct to it
                lda sprirqline,x                ;No, do it normally
                sta rasterpos+3
                ldx #$03
                jmp endraster
r3_last:        lda screenmode                  ;If it's a cutscene, go to the
                cmp #SCRMODE_CUTS               ;"split" interrupt
                beq r3_cuts
                lda $d012                       ;Are we late from RASTER0?
                clc
                adc #$01
                cmp #RASTER0POS
                bcc r3_notlate                  ;No, do it normally
r3_latewait:    lda $d012
                cmp #RASTER0POS
                bcc r3_latewait
                lda savea
                ldx savex
                ldy savey
                jmp raster0                     ;Yes, do it directly
r3_notlate:     ldx #$00
                jmp endraster
r3_cuts:        ldx #$04
                jmp endraster

r3_loady8:      ldx sprphystbl2-8,y
                lda sortspry-8,y
                sta $d001,x
r3_loady7:      ldx sprphystbl2-7,y
                lda sortspry-7,y
                sta $d001,x
r3_loady6:      ldx sprphystbl2-6,y
                lda sortspry-6,y
                sta $d001,x
r3_loady5:      ldx sprphystbl2-5,y
                lda sortspry-5,y
                sta $d001,x
r3_loady4:      ldx sprphystbl2-4,y
                lda sortspry-4,y
                sta $d001,x
r3_loady3:      ldx sprphystbl2-3,y
                lda sortspry-3,y
                sta $d001,x
r3_loady2:      ldx sprphystbl2-2,y
                lda sortspry-2,y
                sta $d001,x
r3_loady1:      ldx sprphystbl2-1,y
                lda sortspry-1,y
                sta $d001,x
r3_loady0:
r3_jmp2:        jmp r3_loadr8


r3_loadr8:      ldx sprphystbl-8,y
                lda sortsprf-8,y
r3_loadf8:      sta SCREEN1+$3f8,x
                lda sortsprc-8,y
                sta $d027,x
                ldx sprphystbl2-8,y
                lda sortsprx-8,y
                sta $d000,x
                lda sortsprxh-8,y
                sta $d010

r3_loadr7:      ldx sprphystbl-7,y
                lda sortsprf-7,y
r3_loadf7:      sta SCREEN1+$3f8,x
                lda sortsprc-7,y
                sta $d027,x
                ldx sprphystbl2-7,y
                lda sortsprx-7,y
                sta $d000,x
                lda sortsprxh-7,y
                sta $d010

r3_loadr6:      ldx sprphystbl-6,y
                lda sortsprf-6,y
r3_loadf6:      sta SCREEN1+$3f8,x
                lda sortsprc-6,y
                sta $d027,x
                ldx sprphystbl2-6,y
                lda sortsprx-6,y
                sta $d000,x
                lda sortsprxh-6,y
                sta $d010

r3_loadr5:      ldx sprphystbl-5,y
                lda sortsprf-5,y
r3_loadf5:      sta SCREEN1+$3f8,x
                lda sortsprc-5,y
                sta $d027,x
                ldx sprphystbl2-5,y
                lda sortsprx-5,y
                sta $d000,x
                lda sortsprxh-5,y
                sta $d010

r3_loadr4:      ldx sprphystbl-4,y
                lda sortsprf-4,y
r3_loadf4:      sta SCREEN1+$3f8,x
                lda sortsprc-4,y
                sta $d027,x
                ldx sprphystbl2-4,y
                lda sortsprx-4,y
                sta $d000,x
                lda sortsprxh-4,y
                sta $d010

r3_loadr3:      ldx sprphystbl-3,y
                lda sortsprf-3,y
r3_loadf3:      sta SCREEN1+$3f8,x
                lda sortsprc-3,y
                sta $d027,x
                ldx sprphystbl2-3,y
                lda sortsprx-3,y
                sta $d000,x
                lda sortsprxh-3,y
                sta $d010

r3_loadr2:      ldx sprphystbl-2,y
                lda sortsprf-2,y
r3_loadf2:      sta SCREEN1+$3f8,x
                lda sortsprc-2,y
                sta $d027,x
                ldx sprphystbl2-2,y
                lda sortsprx-2,y
                sta $d000,x
                lda sortsprxh-2,y
                sta $d010

r3_loadr1:      ldx sprphystbl-1,y
                lda sortsprf-1,y
r3_loadf1:      sta SCREEN1+$3f8,x
                lda sortsprc-1,y
                sta $d027,x
                ldx sprphystbl2-1,y
                lda sortsprx-1,y
                sta $d000,x
                lda sortsprxh-1,y
                sta $d010
r3_loadr0:      jmp r3_sprdone

;Ŀ
;RASTER4                                                                      
;                                                                             
;Raster interrupt for the game/textscreen split in cutscenes.                 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: -                                                                  
;

raster4:        cld                             ;Clear decimal mode
                sta savea
                stx savex
                sty savey
                jsr settextscreen
                pha
                pla
                pha
                pla
                pha
                pla
                nop
                nop
                ldx #$00
                lda #$17
                sta $d011
                jmp endraster

;Ŀ
;WAITRASTER0                                                                  
;                                                                             
;Waits for scorescreen start.                                                 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

waitraster0:    lda #$00
                sta r0count
wr0_loop:       lda r0count
                beq wr0_loop
                rts

;Ŀ
;WAITRASTER1                                                                  
;                                                                             
;Waits for gamescreen start.                                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

waitraster1:    lda #$00
                sta r1count
wr1_loop:       lda r1count
                beq wr1_loop
                rts


