;ͻ
;Metal Warrior 2 - Screen (Text, Scrolling) routines                          
;Started in 8/1999!                                                           
;ͼ

SCRMODE_GAME    = 0
SCRMODE_TEXT    = 1
SCRMODE_LOGO    = 2
SCRMODE_CUTS    = 3
SCRMODE_END     = 4

CI_GROUND       = 1
CI_OBSTACLE     = 2
CI_CLIMB        = 4
CI_OPENDOOR     = 8
CI_MOVINGDOOR   = 16
CI_SIDEDOOR     = 32

SCREEN_START:

;Ŀ
;INITVIDEO                                                                    
;                                                                             
;Sets all necessary VIC registers.                                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initvideo:      lda #SCRMODE_TEXT               ;Set initial screenmode
                sta screenmode
                lda #$00                        ;The only true border color!
                sta screennum                   ;Set gamescreen number
                sta $d020
                sta $d021
                sta $d015                       ;Sprites off
                sta $d01b                       ;Sprites on top of BG
                sta $d017                       ;Sprite Y-expand off
                sta $d01d                       ;Sprite X-expand off
                lda #$ff                        ;Set all sprites multicolor
                sta $d01c
                sta $d026                       ;Set sprite multicolor 2

;Ŀ
;CLEARTEXTSCREEN                                                              
;                                                                             
;Clears the text screen at $a400 and sets colormap to white (multicolor.)     
;                                                                             
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

cleartextscreen:lda #$09                        ;Fill whole screen with
                ldx #$00                        ;multicolor white
cts_loop3:      sta $d800,x
                sta $d900,x
                sta $da00,x
                sta $db00,x
                inx
                bne cts_loop3
                ldx #20                         ;20 rows to do
                lda #$00
                sta destlo
                lda #$a4
                sta desthi
cts_loop1:      ldy #39                         ;40 columns to do
cts_loop2:      lda #$20                        ;Space to screen
                sta (destlo),y
                dey
                bpl cts_loop2
                lda destlo                      ;Go one row down
                clc
                adc #40
                sta destlo
                bcc cts_not1
                inc desthi
cts_not1:       dex                             ;All rows done?
                bne cts_loop1
                rts

;Ŀ
;PRINTTEXTC                                                                   
;                                                                             
;Centered text printing                                                       
;                                                                             
;Parameters: Y:rivi,srclo,srchi                                               
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

printtextc:     tya
                pha
                ldy #$00
                ldx #40
ptc_getlen:     lda (srclo),y
                beq ptc_ready
                dex
                iny
                bne ptc_getlen
ptc_ready:      txa
                lsr
                tax
                pla
                tay

;Ŀ
;PRINTTEXT                                                                    
;                                                                             
;Prints a zero-terminated string                                              
;                                                                             
;Parameters: X,Y,srclo,srchi                                                  
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

printtext:      txa
                clc
                adc screenrowtbllo,y
                sta destlo
                lda #>TEXTSCREEN
                adc screenrowtblhi,y
                sta desthi
                ldy #0
pt_printloop:   lda (srclo),y
                beq pt_printend
                and #$3f
                sta (destlo),y
                iny
                jmp pt_printloop
pt_printend:    iny
                tya
                clc
                adc srclo
                sta srclo
                bcc pt_notover3
                inc srchi
pt_notover3:    rts

;Ŀ
;SCROLLSPRITES                                                                
;                                                                             
;Adjusts the sprite-positionsubtractvalue :-)                                 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

scrollsprites:  ldx #$00
                lda scrollsx
                bpl sspr_not1
                dex
sspr_not1:      stx tempval1
                ldx #$00
                lda scrollsy
                bpl sspr_not2
                dex
sspr_not2:      stx tempval2
                lda scrollsubxl
                clc
                adc scrollsx
                sta scrollsubxl
                lda scrollsubxh
                adc tempval1
                sta scrollsubxh
                lda scrollsubyl
                clc
                adc scrollsy
                sta scrollsubyl
                lda scrollsubyh
                adc tempval2
                sta scrollsubyh
                rts

;Ŀ
;DOSCROLL                                                                     
;                                                                             
;Performs the vile act of scrolling. Should be called just after raster1.     
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

doscroll:       lda r0count                     ;Store value of raster 0
                sta dscr_sawcmp+1               ;counter
                lda #$00                        ;Assume: no screen-shifting
                sta shiftx
                sta shifty
                lda scrollx                     ;Do horizontal movement
                sec
                sbc scrollsx
                sta scrollx
                bpl dscr_notright
                inc shiftx
                jmp dscr_notleft
dscr_notright:  cmp #$08
                bcc dscr_notleft
                dec shiftx
dscr_notleft:   lda scrollx                     ;Keep within limits
                and #$07
                sta scrollx
                lda scrolly                     ;Do vertical movement
                sec
                sbc scrollsy
                sta scrolly
                bpl dscr_notdown
                inc shifty
                jmp dscr_notup
dscr_notdown:   cmp #$08
                bcc dscr_notup
                dec shifty
dscr_notup:     lda scrolly                     ;Keep within limits
                and #$07
                sta scrolly
                lda shiftx                      ;Do we need any shifting?
                ora shifty
                bne dscr_work
                jsr dscr_sawcmp                 ;If not, we're done, but add
                rts                             ;a little delay
dscr_work:      ldx screennum                   ;Get the screen number
                lda dscr_routtbl1lo,x           ;Set the corresponding screen
                sta dscr_jsr1+1                 ;shift routine
                lda dscr_routtbl1hi,x
                sta dscr_jsr1+2
                ldy shiftx
                iny
                tya
                ldx shifty                      ;Get source pos. for screen
                inx                             ;shifting
                clc
                adc dscr_shiftmultbl,x
                pha
                tax
                ldy dscr_shifttbl,x
                ldx #38
dscr_jsr1:      jsr $0000                       ;Perform the screen-shift
                lda blockx                      ;Change the block & map
                clc                             ;positions
                adc shiftx
                bpl dscr_nomapleft
                dec mapx
                jmp dscr_blockxdone
dscr_nomapleft: cmp #$04
                bcc dscr_blockxdone
                inc mapx
dscr_blockxdone:and #$03
                sta blockx
                lda blocky
                clc
                adc shifty
                bpl dscr_nomapup
                dec mapy
                jmp dscr_blockydone
dscr_nomapup:   cmp #$04
                bcc dscr_blockydone
                inc mapy
dscr_blockydone:and #$03
                sta blocky
                pla
                tax
                lda dscr_routtbl2lo,x           ;Set the correct color
                sta dscr_jsr2+1                 ;shift routine
                lda dscr_routtbl2hi,x
                sta dscr_jsr2+2
dscr_jsr2:      jmp $0000                       ;Perform the color shift

dscr_shift1:
N               SET 0
                REPEAT 20
                lda SCREEN1-40+N*40,y
                sta SCREEN2+N*40,x
N               SET N+1
                REPEND
                dey
                dex
                bpl dscr_shift1
dscr_shift1done:rts
dscr_shift2:
N               SET 0
                REPEAT 20
                lda SCREEN2-40+N*40,y
                sta SCREEN1+N*40,x
N               SET N+1
                REPEND
                dey
                dex
                bpl dscr_shift2
dscr_csrts:
dscr_shift2done:rts

dscr_csul:      jsr dscr_drawup
                jsr dscr_drawleft
                jsr dscr_swapandwait
                ldx #37
                ldy #37+41
                jsr dscr_csucommon
                jsr dscr_colorup
                jmp dscr_colorleftbottom

dscr_csucommon: stx dscr_csurestx+1
                sty dscr_csuresty+1
dscr_csuloop1:
N               SET 10
                lda COLORS+N*40,x
                sta colorstore,x
N               SET 9
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS+N*40,y
N               SET N-1
                REPEND
                dey
                dex
                bpl dscr_csuloop1
                jsr dscr_colorup
                lda dscr_csurestx+1
                cmp #37
                bne dscr_csurestx
                jsr dscr_colorlefttop
dscr_csurestx:  ldx #$00
dscr_csuresty:  ldy #$00
dscr_csuloop2:
N               SET 18
                REPEAT 8
                lda COLORS+N*40,x
                sta COLORS+N*40,y
N               SET N-1
                REPEND
                lda colorstore,x
                sta COLORS+N*40,y
                dey
                dex
                bpl dscr_csuloop2
                rts

dscr_csu:       jsr dscr_drawup
                jsr dscr_swapandwait
                ldx #38
                ldy #38+40
                jmp dscr_csucommon

dscr_csur:      jsr dscr_drawup
                jsr dscr_drawright
                jsr dscr_swapandwait
                ldx #1
dscr_csur1:
N               SET 10
                lda COLORS+N*40,x
                sta colorstore,x
N               SET 9
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS+39+N*40,x
N               SET N-1
                REPEND
                inx
                cpx #39
                bcc dscr_csur1
                jsr dscr_colorup
                jsr dscr_colorrighttop
                ldx #1
dscr_csur2:
N               SET 18
                REPEAT 8
                lda COLORS+N*40,x
                sta COLORS+39+N*40,x
N               SET N-1
                REPEND
                lda colorstore,x
                sta COLORS+39+N*40,x
                inx
                cpx #39
                bcc dscr_csur2
                jmp dscr_colorrightbottom

dscr_csl:       jsr dscr_drawleft
                jsr dscr_swapandwait
                ldx #37
dscr_csl1:
N               SET 0
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS+1+N*40,x
N               SET N+1
                REPEND
                dex
                bpl dscr_csl1
                jsr dscr_colorlefttop
                ldx #37
dscr_csl2:
N               SET 10
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS+1+N*40,x
N               SET N+1
                REPEND
                dex
                bpl dscr_csl2
                jmp dscr_colorleftbottom

dscr_csr:       jsr dscr_drawright
                jsr dscr_swapandwait
                ldx #1
dscr_csr1:
N               SET 0
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS-1+N*40,x
N               SET N+1
                REPEND
                inx
                cpx #39
                bcc dscr_csr1
                jsr dscr_colorrighttop
                ldx #1
dscr_csr2:
N               SET 10
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS-1+N*40,x
N               SET N+1
                REPEND
                inx
                cpx #39
                bcc dscr_csr2
                jmp dscr_colorrightbottom

dscr_csdl:      jsr dscr_drawleft
                jsr dscr_drawdown
                jsr dscr_swapandwait
                ldx #37
                ldy #38
                jsr dscr_csdcommon
                jsr dscr_colorleftbottom
                jmp dscr_colordown

dscr_csdcommon: stx dscr_csdrestx+1
                sty dscr_csdresty+1
dscr_csdloop1:
N               SET 1
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS-40+N*40,y
N               SET N+1
                REPEND
                dey
                dex
                bpl dscr_csdloop1
                lda dscr_csdrestx+1
                cmp #37
                bne dscr_csdrestx
                jsr dscr_colorlefttop
dscr_csdrestx:  ldx #$00
dscr_csdresty:  ldy #$00
dscr_csdloop2:
N               SET 11
                REPEAT 9
                lda COLORS+N*40,x
                sta COLORS-40+N*40,y
N               SET N+1
                REPEND
                dey
                dex
                bpl dscr_csdloop2
                rts

dscr_csd:       jsr dscr_drawdown
                jsr dscr_swapandwait
                ldx #38
                ldy #38
                jsr dscr_csdcommon
                jmp dscr_colordown

dscr_csdr:      jsr dscr_drawright
                jsr dscr_drawdown
                jsr dscr_swapandwait
                ldx #1
dscr_csdr1:
N               SET 1
                REPEAT 10
                lda COLORS+N*40,x
                sta COLORS-41+N*40,x
N               SET N+1
                REPEND
                inx
                cpx #39
                bcc dscr_csdr1
                jsr dscr_colorrighttop
                ldx #1
dscr_csdr2:
N               SET 11
                REPEAT 9
                lda COLORS+N*40,x
                sta COLORS-41+N*40,x
N               SET N+1
                REPEND
                inx
                cpx #39
                bcc dscr_csdr2
                jsr dscr_colorrightbottom
                jmp dscr_colordown

dscr_swapandwait:lda screennum                  ;Now swap visible & hidden
                eor #$01                        ;screens
                sta screennum
dscr_sawcmp:    lda #$00                        ;Then wait until raster0count
dscr_saw1:      cmp r0count                     ;has changed from the value
                beq dscr_saw1                   ;at doscroll beginning
                rts

dscr_drawleft:  lda screennum
                eor #$01
                tax
                lda #$00
                sta destlo
                lda screentblhi,x
                sta desthi
                lda #20
                sta destlo2
                ldx mapy
                lda maptbllo,x
                clc
                adc mapx
                sta srclo
                lda maptblhi,x
                adc #$00
                sta srchi
                ldx blockx
                stx dscr_dlinit+1
                lda blocky
                asl
                asl
                adc blockx
dscr_dlstart:   tax
                ldy #$00
                lda (srclo),y
                tay
                lda blocktbllo,y
                sta dscr_dlget+1
                lda blocktblhi,y
                sta dscr_dlget+2
                ldy #$00
dscr_dlget:     lda $1000,x
dscr_dlput:     sta (destlo),y
                lda destlo
                clc
                adc #40
                sta destlo
                bcc dscr_dlnotover1
                inc desthi
dscr_dlnotover1:txa
                clc
                adc #$04
                tax
                cmp #$10
                bcs dscr_dlnewblock
                dec destlo2
                bne dscr_dlget
dscr_dldone:    rts
dscr_dlnewblock:dec destlo2
                beq dscr_dldone
                lda srclo
                clc
                adc mapsizex
                sta srclo
                bcc dscr_dlnotover2
                inc srchi
dscr_dlnotover2:lda (srclo),y
                tax
                lda blocktbllo,x
                sta dscr_dlget+1
                lda blocktblhi,x
                sta dscr_dlget+2
dscr_dlinit:    ldx #$00
                jmp dscr_dlget

dscr_drawright: lda screennum
                eor #$01
                tax
                lda #38
                sta destlo
                lda screentblhi,x
                sta desthi
                lda #20
                sta destlo2
                lda mapx
                clc
                adc #$09
                sta desthi2
                lda blockx
                clc
                adc #$02
                cmp #$04
                bcc dscr_drnotover3
                and #$03
                inc desthi2
dscr_drnotover3:pha
                ldx mapy
                lda maptbllo,x
                clc
                adc desthi2
                sta srclo
                lda maptblhi,x
                adc #$00
                sta srchi
                pla
                sta dscr_dlinit+1
                lda blocky
                asl
                asl
                adc dscr_dlinit+1
                jmp dscr_dlstart

dscr_drawup:    lda screennum
                eor #$01
                tax
                ldy #$00
                sty dscr_duput+1
                lda screentblhi,x
                sta dscr_duput+2
                ldx mapy
                lda maptbllo,x
                clc
                adc mapx
                sta srclo
                lda maptblhi,x
                adc #$00
                sta srchi
                lda blocky
                asl
                asl
                sta dscr_duinit+1
                clc
                adc blockx
                sta destlo2
dscr_dustart:   lda (srclo),y
                tay
                lda blocktbllo,y
                clc
                adc destlo2
                sta dscr_duget+1
                lda blocktblhi,y
                sta dscr_duget+2
                ldx #$00
dscr_duget:     lda $1000
dscr_duput:     sta $1000,x
                inc dscr_duget+1
                lda dscr_duget+1
                and #$03
                beq dscr_dunewblock
                inx
                cpx #39
                bcc dscr_duget
dscr_dudone:    rts
dscr_dunewblock:inx
                cpx #39
                bcs dscr_dudone
                inc srclo
                bne dscr_dunotover1
                inc srchi
dscr_dunotover1:ldy #$00
                lda (srclo),y
                tay
                lda blocktbllo,y
                clc
dscr_duinit:    adc #$00
                sta dscr_duget+1
                lda blocktblhi,y
                sta dscr_duget+2
                jmp dscr_duget

dscr_drawdown:  lda screennum
                eor #$01
                tax
                lda #248
                sta dscr_duput+1
                lda screentblhi,x
                clc
                adc #$02
                sta dscr_duput+2
                lda mapy
                clc
                adc #$04
                tax
                lda maptbllo,x
                clc
                adc mapx
                sta srclo
                lda maptblhi,x
                adc #$00
                sta srchi
                lda blocky
                clc
                adc #$03
                cmp #$04
                bcc dscr_ddnotover2
                and #$03
                pha
                lda srclo
                clc
                adc mapsizex
                sta srclo
                pla
                bcc dscr_ddnotover2
                inc srchi
dscr_ddnotover2:asl
                asl
                sta dscr_duinit+1
                clc
                adc blockx
                sta destlo2
                ldy #$00
                jmp dscr_dustart

dscr_colorrighttop:ldy #38
                bne dscr_colorlefttop2

dscr_colorlefttop: ldy #$00
dscr_colorlefttop2:ldx screennum
                lda screentblhi,x
                sta dscr_clget+2
                lda #>COLORS
                sta dscr_clput+2
dscr_clcommon:  lda #10
                sta destlo
dscr_clget:     ldx SCREEN1,y
                lda CHARCOLORS,x
dscr_clput:     sta $d800,y
                tya
                clc
                adc #40
                tay
                bcc dscr_clnotover
                inc dscr_clget+2
                inc dscr_clput+2
dscr_clnotover: dec destlo
                bne dscr_clget
                rts

dscr_colorrightbottom:ldy #182
                bne dscr_colorleftbottom2

dscr_colorleftbottom: ldy #144
dscr_colorleftbottom2:ldx screennum
                lda screentblhi,x
                clc
                adc #$01
                sta dscr_clget+2
                lda #>COLORS+1
                sta dscr_clput+2
                jmp dscr_clcommon

dscr_colorup:   ldx screennum
                lda screentblhi,x
                sta dscr_cuget+2
                ldy #38
dscr_cuget:     ldx $1000,y
                lda CHARCOLORS,x
                sta $d800,y
                dey
                bpl dscr_cuget
                rts

dscr_colordown: ldx screennum
                lda screentblhi,x
                clc
                adc #$02
                sta dscr_cdget+2
                ldy #38
dscr_cdget:     ldx SCREEN1+19*40,y
                lda CHARCOLORS,x
                sta $d800+19*40,y
                dey
                bpl dscr_cdget
                rts

;Ŀ
;INITMAP                                                                      
;                                                                             
;Initializes the map table of all rows' addresses, and at the same time       
;initializes the block table.                                                 
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

initmap:        ldx #$00
                lda #<MAP
                sta srclo
                lda #>MAP
                sta srchi
                lda #<BLOCKS
                sta destlo
                lda #>BLOCKS
                sta desthi
im_loop:        lda desthi
                sta blocktblhi,x
                lda destlo
                sta blocktbllo,x
                clc
                adc #$10
                bcc im_notover1
                inc desthi
im_notover1:    sta destlo
                lda srchi
                sta maptblhi,x
                lda srclo
                sta maptbllo,x
                clc
                adc mapsizex
                bcc im_notover2
                inc srchi
im_notover2:    sta srclo
                inx
                cpx #$80
                bcc im_loop
                rts

;Ŀ
;SETMAPPOS                                                                    
;                                                                             
;Calculates spritescrolling when mapx,mapy,blockx & blocky have already been  
;set by the caller. Also sets finescrolling!                                  
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X                                                                
;

setmappos:      lda #$07                        ;Set scrolling
                sta scrollx
                sta scrolly
                lda #$00
                sta scrollsubxh
                sta scrollsubyh
                lda mapx                        ;Calculate the sprite-subtract-
                ldx #$05                        ;values
smploop1:       asl
                rol scrollsubxh
                dex
                bne smploop1
                sta scrollsubxl
                lda mapy                        ;Calculate the sprite-subtract-
                ldx #$05
smploop2:       asl                             ;values
                rol scrollsubyh
                dex
                bne smploop2
                sta scrollsubyl
                lda blockx
                asl
                asl
                asl
                sbc #$1e                ;Carry is clear, subtract one more
                clc                     ;automatically
                adc scrollsubxl
                sta scrollsubxl
                bcs smp_notover1
                dec scrollsubxh
smp_notover1:   lda blocky
                asl
                asl
                asl
                sbc #53
                clc
                adc scrollsubyl
                sta scrollsubyl
                bcs smp_notover2
                dec scrollsubyh
smp_notover2:   rts

;Ŀ
;BLANKGAMESCREEN                                                              
;                                                                             
;Blanks out the gamescreen, for redrawing, loading etc.                       
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

blankgamescreen:jsr waitraster0                 ;Wait for scorescreen
                lda #SCRMODE_TEXT               ;Switch blanked text
                sta screenmode                  ;screen display
                jmp cleartextscreen

;Ŀ
;DRAWMAP                                                                      
;                                                                             
;Draws the gamescreen on the current map position.                            
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

drawmap:        ldx screennum
                lda #$00
                sta drm_put+1
                sta drm_put2+1
                lda screentblhi,x
                sta drm_put+2
                lda #>COLORS
                sta drm_put2+2
                lda #20
                sta desthi2
                ldx mapy
                lda maptbllo,x
                clc
                adc mapx
                sta destlo
                lda maptblhi,x
                adc #$00
                sta desthi
                lda blocky
                asl
                asl
                sta drm_init+1
                clc
                adc blockx
                sta destlo2
drm_newline:    lda destlo
                sta srclo
                lda desthi
                sta srchi
                ldy #$00
                lda (srclo),y
                tay
                lda blocktbllo,y
                clc
                adc destlo2
                sta drm_get+1
                lda blocktblhi,y
                sta drm_get+2
                ldx #$00
drm_get:        lda $1000
drm_put:        sta $1000,x
                tay
                lda CHARCOLORS,y
drm_put2:       sta COLORS,x
                inc drm_get+1
                lda drm_get+1
                and #$03
                beq drm_newblock
                inx
                cpx #39
                bcc drm_get
drm_done:       dec desthi2
                bne drm_notdoneyet
                lda #SCRMODE_GAME
                sta screenmode
                rts
drm_notdoneyet: lda drm_put+1
                clc
                adc #40
                sta drm_put+1
                sta drm_put2+1
                bcc drm_notover2
                inc drm_put+2
                inc drm_put2+2
drm_notover2:   lda drm_init+1
                clc
                adc #$04
                and #$0f
                sta drm_init+1
                lda destlo2
                clc
                adc #$04
                pha
                cmp #$10
                bcc drm_notover3
                lda destlo
                clc
                adc mapsizex
                sta destlo
                bcc drm_notover3
                inc desthi
drm_notover3:   pla
                and #$0f
                sta destlo2
                jmp drm_newline
drm_newblock:   inx
                cpx #39
                bcs drm_done
                inc srclo
                bne drm_notover1
                inc srchi
drm_notover1:   ldy #$00
                lda (srclo),y
                tay
                lda blocktbllo,y
                clc
drm_init:       adc #$00
                sta drm_get+1
                lda blocktblhi,y
                sta drm_get+2
                jmp drm_get

;Ŀ
;CHECKMAPLIMITS                                                               
;                                                                             
;Will zero the scrolling speed if at map limits.                              
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

checkmaplimits: lda scrollsx
                beq cml_xok
                bmi cml_xneg
cml_xpos:       lda mapx
                clc
                adc #10
                cmp mapsizex
                bcc cml_xok
                lda blockx
                beq cml_xok
                jmp cml_zerox
cml_xneg:       lda mapx
                bne cml_xok
                lda blockx
                bne cml_xok
cml_zerox:      lda #$00
                sta scrollsx
cml_xok:        lda scrollsy
                beq cml_yok
                bmi cml_yneg
cml_ypos:       lda mapy
                clc
                adc #5
                cmp mapsizey
                bcc cml_yok
                jmp cml_zeroy
cml_yneg:       lda mapy
                bne cml_yok
                lda blocky
                bne cml_yok
cml_zeroy:      lda #$00
                sta scrollsy
cml_yok:        rts

;Ŀ
;GETCHARPOS                                                                   
;                                                                             
;Selvitt hahmon kartta-position taustan tutkimista varten.                  
;Tm rutiini on suhteellisen hidas.                                          
;                                                                             
;Parametrit: X:hahmon numero                                                  
;Palauttaa: -                                                                 
;Modifioi: A,Y                                                                
;

getcharpos:     lda actyh,x
                sta tempval2
                lda actyl,x                     ;Jaetaan Y-koord. kahdeksalla
                lsr tempval2
                ror
                lsr tempval2
                ror
                lsr tempval2
                ror
                sta tempval1                 ;Merkkipositio talteen
                and #3
                sta charblocky
                lda tempval1
                ldy tempval2
                lsr                             ;Jaetaan merkkipos. neljll
                lsr                             ;karttaposition saamiseksi
                clc
                adc rowhitbl,y                  ;(listn yltavu)
                sta charmapy
                lda actxh,x
                sta tempval2
                lda actxl,x                   ;Jaetaan X-koord. kahdeksalla
                lsr tempval2
                ror
                lsr tempval2
                ror
                lsr tempval2
                ror
                sta tempval1                 ;Merkkipositio talteen
                and #3
                sta charblockx
                lda tempval1
                ldy tempval2
                lsr                             ;Jaetaan merkkipos. neljll
                lsr                             ;karttaposition saamiseksi
                clc
                adc rowhitbl,y                  ;(listn yltavu)
                sta charmapx
                rts


;Ŀ
;Rutiinit merkintutkintapaikan siirtmiseen 1:ll merkill                    
;                                                                             
;Parametrit: -                                                                
;Palauttaa: -                                                                 
;Modifioi: A                                                                  
;

movecharposright1:
                inc charblockx
                lda charblockx
                cmp #4
                bcs mcpr1_over
                rts
mcpr1_over:     lda #0
                sta charblockx
                inc charmapx
                rts

movecharposleft1:
                dec charblockx
                bmi mcpl1_over
                rts
mcpl1_over:     lda #3
                sta charblockx
                dec charmapx
                rts

movecharposdown1:
                inc charblocky
                lda charblocky
                cmp #4
                bcs mcpd1_over
                rts
mcpd1_over:     lda #0
                sta charblocky
                inc charmapy
                rts

movecharposup1: dec charblocky
                bmi mcpu1_over
                rts
mcpu1_over:     lda #3
                sta charblocky
                dec charmapy
                rts

;Ŀ
;GETCHARINFO                                                                  
;                                                                             
;Selvitt merkki-infon paikasta (charmapx,charmapy):(charblockx,charblocky)  
;Tm rutiini on kohtalaisen nopea!                                           
;                                                                             
;Parametrit: -                                                                
;Palauttaa: A:merkki-info                                                     
;Modifioi: A,Y                                                                
;

getcharinfo:    ldy charmapy
                cpy mapsizey
                bcs gci_out
                lda maptbllo,y
                sta tempval1
                lda maptblhi,y
                sta tempval2
                ldy charmapx
                cpy mapsizex
                bcs gci_out
                lda (tempval1),y              ;Otetaan blokki
                tay
                lda blocktbllo,y
                sta tempval1
                lda blocktblhi,y
                sta tempval2
                lda charblocky
                asl
                asl
                adc charblockx
                tay
                lda (tempval1),y                ;Otetaan merkki blokista
                tay
                lda CHARINFO,y                  ;Otetaan merkki-info
                rts
gci_out:        lda #CI_OBSTACLE                ;Checking from outside the
                rts                             ;map results in an obstacle

