;ͻ
;Metal Warrior 2 - Sprite & Sprite Multiplexing routines                      
;Started in 9/1999!                                                           
;ͼ

SPRITE_START:

;Ŀ
;INITSPRITES                                                                  
;                                                                             
;Clears all sprites & resets sprite-doublebuffering.                          
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;

initsprites:    sei
                lda #$00
                sta sprirqamount
                sta sprirqamount+MAXSPRIRQ
                sta r1sprirqturn
                sta numspr
                cli
                rts

;Ŀ
;GETPLAYERCOLORS                                                              
;                                                                             
;Sets colors of sprites $40-$8f (Player & common sprites). This is the only   
;time when RAM under the IO chips has to be accessed.                         
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

getplayercolors:sei
                lda $d011
                and #$ef
                sta $d011
                lda #$34
                sta $01
                lda #<PLAYERSPRITES
                sta srclo
                lda #>PLAYERSPRITES
                sta srchi
                ldx #$40
cps_loop1:      ldy #$3f
                lda (srclo),y
                sta sprcolortbl,x
                lda srclo
                clc
                adc #$40
                sta srclo
                bcc cps_notover1
                inc srchi
cps_notover1:   inx
                cpx #$90
                bcc cps_loop1
cps_rest01:     lda #$37
                sta $01
                cli
                rts

;Ŀ
;GETENEMYCOLORS                                                               
;                                                                             
;Gets colors of sprites $90-$fe after loading.                                
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

getenemycolors: lda #<LEVELSPRITES
                sta srclo
                lda #>LEVELSPRITES
                sta srchi
                ldx #$90
                ldy #$3f
gec_loop:       lda (srclo),y
                sta sprcolortbl,x
                lda srclo
                clc
                adc #$40
                sta srclo
                bcc gec_notover1
                inc srchi
gec_notover1:   inx
                bne gec_loop
                lda #$01                        ;This is the "flashing" color
                sta sprcolortbl+$ff
                rts

;Ŀ
;DOSPRITES                                                                    
;                                                                             
;Sorts the sprites for the double-buffered spriteirq system. Maximum 24       
;sprites. Uses quite a bit of tables; no sacrifice is too big for speed!      
;                                                                             
;Parameters: numspr                                                           
;Returns: -                                                                   
;Modifies: A,X,Y                                                              
;

dosprites:      ldx #$00
                stx sprend
dspr_waitswitch:lda r1sprirqturn                ;Check side of doublebuffer
                bmi dspr_waitswitch             ;If switch still in progress,
dspr_noswitch:  bne dspr_noturn2                ;wait until it's over
                ldx #MAXSPRIRQ
dspr_noturn2:   stx numsprirq
                txa
                clc
                adc #MAXSPRIRQ-1
                sta lastsprirq
                ldy numspr                      ;Are there sprites at all?
                bne dspr_cont
                jmp dspr_done
dspr_cont:      dey
                lda #$00
N               SET 0                           ;First clear sprite amounts
                REPEAT SPRROWS                  ;on all rows
                sta sprrow_a+N
N               SET N+1
                REPEND
dspr_loop1:     lda spryh,y                     ;Check high Y-coord
                bne dspr_skip1
                lda sprxh,y                     ;Check high X-coord
                beq dspr_xok1
                cmp #$01
                bne dspr_skip1
                lda sprxl,y
                cmp #79
                bcs dspr_skip1
dspr_xok1:      ldx spryl,y                     ;Take Y-coord of sprite
                lda sprrowtbl,x                 ;Convert to row
                bmi dspr_skip12                 ;If negative, sprite out of
                sta sprrow,y                    ;screen
                inc sprend
                tax
                inc sprrow_a,x                  ;Increase amount of sprites on
                dey                             ;row, go on to next sprite
                bpl dspr_loop1
                jmp dspr_phase1done
dspr_skip1:     lda #$ff
dspr_skip12:    sta sprrow,y
                dey
                bpl dspr_loop1
dspr_phase1done:lda sprend                      ;Any sprites onscreen?
                bne dspr_cont2
                jmp dspr_done
dspr_cont2:     lda #$00                        ;Which side of the sprite-irq
                ldx r1sprirqturn                ;"doublebuffer" are we using?
                bne dspr_noturn1                ;(we must now use the "hidden"
                lda #MAXSPR                     ;side)
dspr_noturn1:   sta sprstart
                clc
                adc sprend
                sta sprend
                lda sprstart
                clc                             ;Now use an unrolled loop to
N               SET 0                           ;calculate the first free
                sta sprrow_first+N              ;index on each row
                REPEAT SPRROWS-1
                adc sprrow_a+N
                sta sprrow_first+1+N
N               SET N+1
                REPEND
                ldy #$00
dspr_loop2:     ldx sprrow,y                    ;Now do a second loop to
                bmi dspr_skip2                  ;copy the sprites to the
                lda sprrow_first,x              ;sorted table
                inc sprrow_first,x
                tax
                lda spryl,y
                sta sortspry,x
                lda sprxl,y
                sta sortsprx,x
                lda sprxh,y
                sta sortsprxh,x
                lda sprf,y                      ;Process color
                sta sortsprf,x
                ora sprfls,y                    ;Process flashing
                sta dspr_getcolor+1
dspr_getcolor:  lda sprcolortbl
                sta sortsprc,x
dspr_skip2:     iny
                cpy numspr
                bcc dspr_loop2
dspr_sortdone:  ldx sprstart                    ;X=first logical sprite
                stx thisirqstart
                txa
                clc
                adc #$08
                sta dspr_ficmp+1
dspr_firstirq:  ldy sprphystbl,x
                lda sortsprxh,x                 ;Process X-coord MSB
                beq dspr_xlow2
dspr_xhigh2:    lda tempd010
                ora sprortbl,y
                bne dspr_xdone2
dspr_xlow2:     lda tempd010
                and sprandtbl,y
dspr_xdone2:    sta tempd010
                sta sortsprxh,x
                inx
dspr_ficmp:     cpx #$00                        ;Max. 8 sprites in one
                bcs dspr_firstirqdone           ;interrupt
                cpx sprend                      ;All sprites done?
                bcc dspr_firstirq
dspr_firstirqdone:
                jmp dspr_irqdone
dspr_irqloop:   lda sortspry-8,x                ;Now find first suitable
                clc
                adc #20
                cmp sortspry,x                  ;sprite for this spriteirq
                bcc dspr_found
                inx                             ;Keep looking...
                cpx sprend                      ;until all logical sprites
                bcc dspr_irqloop                ;done
                jmp dspr_done
dspr_found:     lda #$ff
                sta thisirqline
                stx thisirqstart
                txa                             ;Note: Carry=0
                adc #$08                        ;Last logical sprite which
                sta dspr_endcmp+1               ;can be loaded in the same irq
                lda thisirqline
dspr_sprloop:   lda sortspry,x                  ;Higher than the previous?
                cmp thisirqline
                bcs dspr_nothigher
                sta thisirqline
dspr_nothigher: ldy sprphystbl,x
                lda sortsprxh,x                 ;Process X-coord MSB
                beq dspr_xlow
dspr_xhigh:     lda tempd010
                ora sprortbl,y
                bne dspr_xdone
dspr_xlow:      lda tempd010
                and sprandtbl,y
dspr_xdone:     sta tempd010
                sta sortsprxh,x
                inx                             ;Next logical sprite
dspr_endcmp:    cpx #$00                        ;Max. 8 sprites in one
                bcs dspr_irqdone                ;interrupt
                cpx sprend                      ;All sprites done?
                bcs dspr_irqdone
                lda sortspry-7,x                ;Can the next physical sprite
                cmp sortspry-8,x                ;be overwritten
                bcc dspr_sprloop
                lda sortspry-8,x
                adc #19
                cmp thisirqline
                bcc dspr_sprloop
dspr_irqdone:   txa
                sec
                sbc thisirqstart
                sta thisirqamount
                tay
                lda thisirqline
                sec
                sbc sprlinesubtbl,y
                ldy numsprirq
                sta sprirqline,y
                lda thisirqamount
                sta sprirqamount,y
                clc
                adc thisirqstart
                sta sprirqstart,y
                iny                             ;Now increase irq-num.
                sty numsprirq
                cpy lastsprirq                  ;No more irqs?
                bcs dspr_done
                cpx sprend                      ;Still sprites undone?
                bcs dspr_done
                jmp dspr_irqloop
dspr_done:      ldy numsprirq                   ;Now set the amount of sprites
                lda #$00                        ;in the first unused spr-irq
                sta sprirqamount,y              ;to 0 as a stop signal
                rts



;Ŀ
;SWITCHSPRITES                                                                
;                                                                             
;Switches the "visible" half of the sprite-doublebuffer. Call this when you   
;want the new sprites shown on the next screen update.                        
;                                                                             
;Parameters: -                                                                
;Returns: -                                                                   
;Modifies: A                                                                  
;
switchsprites:  lda r1count                     ;Wait until raster1-count
swspr_loop:     cmp r1count                     ;increases
                beq swspr_loop
                lda r1sprirqturn                ;New sprites will be visible
                eor #$01                        ;on the next screen update
                ora #$80
                sta r1sprirqturn
                rts

